/*

					Oscilloscope AVR

			Copyrights 2008,  Yoshi Hiroshi

				ADC Block - ATmega644P
*/

#include <avr/io.h>

#include "OscilloAdc.h"
#define __AVR_ADC_INCLUDED__
#include "CmndMsg.h"
#include "Command.h"
#include "History.h"

volatile unsigned char commandStatus;
volatile unsigned char polarity_switch; // 0:Plus, 1:Minus
volatile unsigned char signed_switch;   // 0:NoSigned 1:Signed
volatile unsigned char gain_switch;
volatile unsigned char range_switch;
volatile unsigned char input_switch;
volatile unsigned char triggerMode;
volatile unsigned char triggerLevel;
volatile unsigned char samplingInited;

//---------------------------------------------------------------------
//
//				CommandInit
//
//---------------------------------------------------------------------
void CommandInit()
{
	commandStatus = COMMAND_STATUS_IDLE;
	gain_switch = 0;
	range_switch = 0x01;
	input_switch = 0x00;
	triggerMode = 0xFF;
	samplingInited = SAMPLING_INITED;
	ControlPortOut();
}

//---------------------------------------------------------------------
//
//				ControlPortOut
//
//---------------------------------------------------------------------
void ControlPortOut()
{
	OSCILLO_CONTROL_PORT = range_switch | (input_switch << OSCILLO_SWITCH_0);
}


void TriggerLevelInit()
{
	triggerLevel = 0x80;
}

//---------------------------------------------------------------------
//
//				DoCommand
//
//---------------------------------------------------------------------
void DoCommand(unsigned char c)
{
	if(commandStatus == COMMAND_STATUS_IDLE)
	{
		switch(c)
		{
		case ADC_COMMAND_CODE_NOP:
			break;
		case  ADC_COMMAND_CODE_GET_DEVICE:
			GetDeviceType();
			break;
		case ADC_COMMAND_CODE_CLEARALL:
			ClearAll();
			break;

		case ADC_COMMAND_CODE_SET_CHANNEL_RANGE:
			commandStatus = COMMAND_STATUS_SET_RANGE;
			break;
		case ADC_COMMAND_CODE_SET_CHANNEL_POSITION:
			commandStatus = COMMAND_STATUS_SET_POSITION;
			break;
		case ADC_COMMAND_CODE_SET_CHANNEL_TRG_MODE:
			commandStatus = COMMAND_STATUS_SET_TRIGGER_MODE;
			break;
		case ADC_COMMAND_CODE_SET_CHANNEL_TRG_LEVEL:
			commandStatus = COMMAND_STATUS_SET_TRIGGER_LEVEL;
			break;
		case ADC_COMMAND_CODE_SET_CHANNEL_SWITCH:
			commandStatus = COMMAND_STATUS_SET_SWITCH;
			break;

		case ADC_COMMAND_CODE_FETCH_FIRST:
			HistoryGetFirst();
			break;
		case ADC_COMMAND_CODE_FETCH_NEXT:
			HistoryGetNext();
			break;
		case ADC_COMMAND_CODE_FETCH_NUM_DATA_H:
			HistoryGetLengthH();
			break;
		case ADC_COMMAND_CODE_FETCH_NUM_DATA_L:
			HistoryGetLengthL();
			break;
		}
	} else {
		switch(commandStatus)
		{
		case COMMAND_STATUS_SET_RANGE:
			SetChannelRange(c);
			commandStatus = COMMAND_STATUS_IDLE;
			break;
		case COMMAND_STATUS_SET_POSITION:
			SetChannelPosition(c);
			commandStatus = COMMAND_STATUS_IDLE;
			break;
		case COMMAND_STATUS_SET_TRIGGER_MODE:
			SetChannelTriggerMode(c);
			commandStatus = COMMAND_STATUS_IDLE;
			break;
		case COMMAND_STATUS_SET_TRIGGER_LEVEL:
			SetChannelTriggerLevel(c);
			commandStatus = COMMAND_STATUS_IDLE;
			break;
		case COMMAND_STATUS_SET_SWITCH:
			SetChannelSwitch(c);
			commandStatus = COMMAND_STATUS_IDLE;
			break;
		}
	}
}


//---------------------------------------------------------------------
//
//		Get Device Type
//
//---------------------------------------------------------------------
void GetDeviceType()
{
	SET_SPI_DATA(DEVICE_TYPE_ATmega644P);
}

//---------------------------------------------------------------------
//
//		Clear All
//
//---------------------------------------------------------------------
void ClearAll()
{
	History_Init();
	RESET_TRIGGER_DETECTED;
	samplingInited = SAMPLING_INITED;
	triggerInited = TRIGGER_NOT_INITED;
}

//---------------------------------------------------------------------
//
//		SetChannelRange
//
//---------------------------------------------------------------------
void SetChannelRange(unsigned char val)
{
	range_switch = val & 0x0F;
	gain_switch = (val >> 4) & 0x07;
	ControlPortOut();
	switch(gain_switch)
	{
	case 0:
		adcChannelControl =  _BV(ADLAR) | OSCILLO_ADC_DATA_x1_0_1;	// x1 diff
		polarity_switch = POLARITY_POSITIVE;
		signed_switch = ADC_DATA_SIGNED;
		break;
	case 1:
		adcChannelControl =  _BV(ADLAR) | OSCILLO_ADC_DATA_x10_1_0;	// x10 diff
		polarity_switch = POLARITY_NEGATIVE;
		signed_switch = ADC_DATA_SIGNED;
		break;
	case 2:
		adcChannelControl =  _BV(ADLAR) | OSCILLO_ADC_DATA_x200_1_0; // x200 diff
		polarity_switch = POLARITY_NEGATIVE;
		signed_switch = ADC_DATA_SIGNED;
		break;
	case 3:
		adcChannelControl =  _BV(ADLAR) | OSCILLO_ADC_DATA_x1_0;	// x1 single ADC0
		polarity_switch = POLARITY_POSITIVE;
		signed_switch = ADC_DATA_UNSIGNED;
		break;
	case 4:
		adcChannelControl =  _BV(ADLAR) | OSCILLO_ADC_DATA_x1_1_1; // x1 diff offset
		polarity_switch = POLARITY_POSITIVE;
		signed_switch = ADC_DATA_SIGNED;
		break;
	case 5:
		adcChannelControl =  _BV(ADLAR) | OSCILLO_ADC_DATA_x10_0_0; // x10 diff offset
		polarity_switch = POLARITY_NEGATIVE;
		signed_switch = ADC_DATA_SIGNED;
		break;
	case 6:
		adcChannelControl =  _BV(ADLAR) | OSCILLO_ADC_DATA_x200_0_0; // x200 diff offset
		polarity_switch = POLARITY_NEGATIVE;
		signed_switch = ADC_DATA_SIGNED;
		break;
	case 7:
		adcChannelControl =  _BV(ADLAR) | OSCILLO_ADC_DATA_x1_1;	// x1 single ADC1
		polarity_switch = POLARITY_POSITIVE;
		signed_switch = ADC_DATA_UNSIGNED;
		break;
	}
	if(ampPolarity == POLARITY_NEGATIVE)
	{
		if(polarity_switch == POLARITY_POSITIVE)
		{
			polarity_switch = POLARITY_NEGATIVE;
		}
		else
		{
			polarity_switch = POLARITY_POSITIVE;
		}
	}
	if(val & 0x80)
	{
		if(polarity_switch == POLARITY_POSITIVE)
		{
			polarity_switch = POLARITY_NEGATIVE;
		}
		else
		{
			polarity_switch = POLARITY_POSITIVE;
		}
	}
	ADMUX = adcChannelControl;
	RESET_TRIGGER_DETECTED;
}

//---------------------------------------------------------------------
//
//		SetChannelPosition
//
//---------------------------------------------------------------------
void SetChannelPosition(unsigned char val)
{
	unsigned char offset;
	switch(gain_switch)
	{
	case 0:
	case 3:
	case 4:
	case 7:
		OCR2A = val;
		break;
	case 1:
	case 2:
	case 5:
	case 6:
		if(val >= 0x80)
		{
			offset = val - 0x80;
			offset = offset >> 2;
			val = 0x80 + offset;
		} else {
			offset = 0x80 - val;
			offset = offset >> 2;
			val = 0x80 - offset;
		}
		OCR2A = val;
		break;
	}
}

//---------------------------------------------------------------------
//
//		SetChannelTriggerMode
//
//---------------------------------------------------------------------
void SetChannelTriggerMode(unsigned char val)
{
	if(val == 0x10)
	{
		triggerMode = TRIGGERMODE_SLOPE_PLUS;
	}
	if(val == 0x11)
	{
		triggerMode = TRIGGERMODE_SLOPE_MINUS;
	}
	RESET_TRIGGER_DETECTED;
}

//---------------------------------------------------------------------
//
//		SetChannelTriggerLevel
//
//---------------------------------------------------------------------
void SetChannelTriggerLevel(unsigned char val)
{
	triggerLevel = val;
	RESET_TRIGGER_DETECTED;
}


//---------------------------------------------------------------------
//
//		SetChannelSwitch
//
//---------------------------------------------------------------------
void SetChannelSwitch(unsigned char val)
{
	input_switch = val & 0x0F;
	ControlPortOut();
}
