/*

					Oscilloscope AVR

			Copyrights 2008,  Yoshi Hiroshi

				ADC Block - ATmega644P
*/

#include <avr/io.h>

#include "OscilloAdc.h"
#include "History.h"

unsigned char* histHead;
unsigned char* histBottom;
unsigned char* histReadPtr;
unsigned char histBuff[HISTORY_BUFFER_SIZE];

//---------------------------------------------------------------------
//
//		Get Next Pointer
//
//---------------------------------------------------------------------
unsigned char* GetNextPointer(unsigned char* p)
{
	p++;
	if(p >= (histBuff + HISTORY_BUFFER_SIZE))
	p = histBuff;
	return p;
}


//---------------------------------------------------------------------
//
//		Initialize History
//
//---------------------------------------------------------------------
void History_Init()
{
	histHead = histBottom = histBuff;
	histReadPtr = histHead;
//	for(int index = 0; index < sizeof(histBuff); histBuff[index++]=0);
}

//---------------------------------------------------------------------
//
//		Append History Data into History Buffer & advance pointers
//
//---------------------------------------------------------------------
void AppendHistory(unsigned char d)
{
	*histHead = d;
	histHead = GetNextPointer(histHead);
	if(histHead == histBottom) histBottom = GetNextPointer(histBottom);
}

//---------------------------------------------------------------------
//
//		Get First History Pointer & Data
//
//---------------------------------------------------------------------
void HistoryGetFirst()
{
	histReadPtr = histBottom;
	SET_SPI_DATA(*histReadPtr);
}

//---------------------------------------------------------------------
//
//		Get Next History Pointer & Data
//
//---------------------------------------------------------------------
void HistoryGetNext()
{
	histReadPtr = GetNextPointer(histReadPtr);
	SET_SPI_DATA(*histReadPtr);
}

//---------------------------------------------------------------------
//
//		Get Length of History Data
//
//---------------------------------------------------------------------
int HistoryGetLength()
{
	int length;
	if(histBottom > histHead)
	{
		length = (histBuff + HISTORY_BUFFER_SIZE - histBottom) + (histHead - histBuff);
	} else {
		length = histHead - histBottom;
	}
	return length;
}

//---------------------------------------------------------------------
//
//		Get Length of History Data (Higher 8 bit)
//
//---------------------------------------------------------------------
void HistoryGetLengthH()
{
	SET_SPI_DATA(HistoryGetLength() >> 8);
}

//---------------------------------------------------------------------
//
//		Get Length of History Data (Lower 8 bit)
//
//---------------------------------------------------------------------
void HistoryGetLengthL()
{
	SET_SPI_DATA(HistoryGetLength() & 0xFF);
}
