/*

					Oscilloscope AVR

			Copyrights 2008,  Yoshi Hiroshi

				ADC Block - ATmega644P
*/

#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>
#include "OscilloAdc.h"
#include "Command.h"
#include "History.h"

volatile unsigned char ampPolarity;
unsigned char adcData;
unsigned char adcValue;

volatile unsigned char spiCommand;
volatile unsigned char spiRecvData;
volatile unsigned char spiDataSave;
volatile char triggerInited;
volatile char triggerInitialLevel;



//---------------------------------------------------------------------
//
//		SPI Serial Transfer Complete Interrupt
//
//---------------------------------------------------------------------
ISR(SPI_STC_vect)
{
	spiInterruptFlag = 1;
	spiRecvData = SPDR;
	RECOVER_SPI_DATA;
}

//---------------------------------------------------------------------
//
//		ADC Complete Interrupt
//
//---------------------------------------------------------------------
ISR(ADC_vect, ISR_NAKED)
{
	DEBUG_END_ADC_CONV;
	reti();
}

//---------------------------------------------------------------------
//
//		External Interrupt 2	Start AD-Conversion
//
//---------------------------------------------------------------------
ISR(INT2_vect, ISR_NAKED)
{
//	asm volatile("push r24");
	ADMUX =  adcChannelControl;	// ADC͐ݒ
	ADCSRA = _BV(ADSC) |		// ADϊJn
			 _BV(ADIF) | 
			 ADC_START_COMMAND;
	DEBUG_BEGIN_ADC_CONV;		// fobOp
	adcStartInterruptFlag |= 1;	// tOZbg
//	asm volatile("pop r24");
	reti();
}

//---------------------------------------------------------------------
//
//		main
//
//---------------------------------------------------------------------
int main()
{
	ADCSRA = ADC_START_COMMAND;
	adcData = 0;
	ampPolarity = POLARITY_POSITIVE;	// Positive
//	ampPolarity = POLARITY_NEGATIVE;	// Negative
	adcStartInterruptFlag = 0;
	spiInterruptFlag = 0;

	DEBUG_PORT_SETUP;
	OSCILLO_CONTROL_DDR = 0xFF;
	OSCILLO_CONTROL_PORT = 0x00;

	// Position Value Output
	OCR2A = 0x80;
	TCCR2A = _BV(COM2A1) | _BV(COM2A0) |	// l傫ƂAd
//	TCCR2A = _BV(COM2A1) |					// lƂAd
			 _BV(WGM21) | _BV(WGM20);
	TCCR2B = _BV(CS20);

	// ADC_START Setup
	EICRA = _BV(ISC21) | _BV(ISC20);
	EIMSK = _BV(INT2);

	// SPI & DEBUG configuration
	DDRB =	_BV(TRIGGER_DETECT) |
			_BV(SPI_MISO);
	PORTB = _BV(ADC_START) |
			_BV(TRIGGER_DETECT) |
			_BV(SPI_SS) |
			_BV(SPI_MOSI) |
			_BV(SPI_SCK);
	SPCR = _BV(SPIE) | _BV(SPE);

	CommandInit();
	History_Init();
	adcChannelControl = _BV(ADLAR) | OSCILLO_ADC_DATA_x1_0;
	ADMUX =  adcChannelControl;

	while(1)
	{
		sleep_enable();	// X[vɈڍŝ
		sei();			// 
		sleep_cpu();	// 	X[vāA̔҂
		cli();			// ֎~
		sleep_disable();	// X[vւ̈ڍs֎~

		if(spiInterruptFlag)	// SPItO̓ZbgĂ邩H
		{
			spiInterruptFlag = 0;	// tONA
			DoCommand(spiRecvData);
		}

		if(adcStartInterruptFlag)	// tO̓ZbgĂ邩H
		{
			adcStartInterruptFlag = 0;	// tONA
			if(samplingInited != SAMPLING_INITED)			// ŏ̃f[^͓ǂݎ̂
			{
				// Of[^rohőMł悤ɂ
				DEBUG_BEGIN_GOT_DATA;
				SET_SPI_DATA(adcData);
				DEBUG_END_GOT_DATA;

				// Av̋ɐADC̋ɐŐM̐𔽓]
				// Q̕␔ƂƁA̍őlŃI[o[t[̂ŁA
				// P̕␔ƂBmɂ́APkrǎ덷B
				if(polarity_switch == POLARITY_NEGATIVE)		// ADC Negative
				{
					adcData = ~ADCH;
				} 
				else 
				{				// ADC Positive
					adcData = ADCH;
				}

				// gK[opf[^̎Zo
				// ̗Lɂ炸AɂB
				if(signed_switch == ADC_DATA_SIGNED)		// t
				{
					adcValue = adcData + 0x80;
				}
				else					// 
				{
					adcValue = adcData;
				}

				// gK[
				if(triggerInited == TRIGGER_NOT_INITED)
				{
					if(triggerMode == TRIGGERMODE_SLOPE_PLUS) // Plus
					{
						if(adcValue > triggerLevel)
						{		// 擾lgK[x𒴂H
							if(triggerInitialLevel == TRIGGER_INITIAL_LEVEL_LOWER)
							{ 	// LOWHIGHJڂŃgK[
						 		TRIGGER_DETECTED;
								triggerInitialLevel = TRIGGER_INITIAL_LEVEL_HIGHER;
							}
						} 
						else 
						{
							triggerInitialLevel = TRIGGER_INITIAL_LEVEL_LOWER;
						}
					}
					if(triggerMode == TRIGGERMODE_SLOPE_MINUS) // Minus
					{
						if(adcValue < triggerLevel)
						{		// 擾lgK[xH
							if(triggerInitialLevel == TRIGGER_INITIAL_LEVEL_HIGHER)
							{ 	// HIGHLOWJڂŃgK[
						 		TRIGGER_DETECTED;
								triggerInitialLevel = TRIGGER_INITIAL_LEVEL_LOWER;
							}
						} 
						else 
						{
							triggerInitialLevel = TRIGGER_INITIAL_LEVEL_HIGHER;
						}
					}
				} 
				else 
				{		// ŏ̃f[^̓gK[̏l肷
					triggerInited = TRIGGER_INITED;
					triggerInitialLevel = (adcValue >= triggerLevel);
				}

				// f[^𗚗obt@Ɋi[
				// i[f[^̓AŏɊi[ꂽf[^̓S~
				DEBUG_BEGIN_SAVE_DATA;
				AppendHistory(spiDataSave);
				DEBUG_END_SAVE_DATA;
			} 
			else 
			{		// ŏADC_STARTM͕ϊJnȂ̂ŁAȂB
				samplingInited = SAMPLING_NOT_INITED;
			}
		}
	}
}
