/*

					Oscilloscope AVR

			Copyrights 2008,  Yoshi Hiroshi

				ADC Block - ATmega644P
*/

#define TRIGGER_NOT_INITED						0
#define TRIGGER_INITED							1

#define TRIGGER_INITIAL_LEVEL_LOWER				0
#define TRIGGER_INITIAL_LEVEL_HIGHER			1

//---------------------------------------------------------------------
//
//			Global Storage
//
//---------------------------------------------------------------------
extern volatile unsigned char ampPolarity;
extern volatile unsigned char spiDataSave;
extern volatile char triggerInited;


//---------------------------------------------------------------------
//
//		Analog Block Control Definitions
//
//---------------------------------------------------------------------
#define OSCILLO_CONTROL_PORT				PORTD
#define OSCILLO_CONTROL_PIN					PIND
#define OSCILLO_CONTROL_DDR					DDRD
#define OSCILLO_POSITION						7
#define OSCILLO_SWITCH_2						6
#define OSCILLO_SWITCH_1						5
#define OSCILLO_SWITCH_0						4
#define OSCILLO_RANGE_1_1						3
#define OSCILLO_RANGE_1_0						2
#define OSCILLO_RANGE_0_1						1
#define OSCILLO_RANGE_0_0						0

//---------------------------------------------------------------------
//
//		ADC Definitions
//
//---------------------------------------------------------------------
//ADC Definitions
#define OSCILLO_ADC_DATA_x1_0					(0)
#define OSCILLO_ADC_DATA_x1_1					(1)
#define OSCILLO_ADC_DATA_x1_2					(2)
#define OSCILLO_ADC_DATA_x1_0_1					(16)
#define OSCILLO_ADC_DATA_x1_1_1					(17)
#define OSCILLO_ADC_DATA_x10_0_0				(8)
#define OSCILLO_ADC_DATA_x10_1_0				(9)
#define OSCILLO_ADC_DATA_x200_0_0				(10)
#define OSCILLO_ADC_DATA_x200_1_0				(11)

//#define ADC_START_COMMAND						(_BV(ADEN) | _BV(ADPS2))
#define ADC_START_COMMAND						(_BV(ADEN) | _BV(ADIE) | _BV(ADPS2))

#define adcStartInterruptFlag				GPIOR0
#define adcChannelControl					GPIOR1
#define spiInterruptFlag					GPIOR2


//---------------------------------------------------------------------
//
//		SPI Definitions & PORT B Setting
//
//---------------------------------------------------------------------
#define ADC_START								PB2		// INPUT
#define TRIGGER_DETECT							PB3		// OUTPUT
#define SPI_SS									PB4		// INPUT
#define SPI_MOSI								PB5		// INPUT
#define SPI_MISO								PB6		// OUTPUT
#define SPI_SCK									PB7		// INPUT

#define TRIGGER_DETECTED					PORTB &= ~_BV(TRIGGER_DETECT)
#define RESET_TRIGGER_DETECTED				PORTB |= _BV(TRIGGER_DETECT)
#define SET_SPI_DATA(d)						SPDR = d; spiDataSave = d
#define SAVE_SPI_DATA(d)					spiDataSave = d
#define RECOVER_SPI_DATA					SPDR = spiDataSave

//---------------------------------------------------------------------
//
//		Debug PORT Setting
//
//---------------------------------------------------------------------
#define DEBUG_PORT_SETUP					DDRC |= 0x3F
#define DEBUG_PORT							PORTC


#define DEBUG_ADC_CONVERSION					PC0		// OUTPUT
#define DEBUG_GOT_ADC_DATA						PC1		// OUTPUT
#define DEBUG_SAVE_DATA_INTO_BUFFER				PC2		// OUTPUT
#define DEBUG_RANGE_GAIN0						PC3		// OUTPUT
#define DEBUG_RANGE_GAIN1						PC4		// OUTPUT

#define DEBUG_BEGIN(bit)					DEBUG_PORT |= _BV(bit)
#define DEBUG_END(bit)						DEBUG_PORT &= ~_BV(bit)

#define DEBUG_BEGIN_ADC_CONV				DEBUG_BEGIN(DEBUG_ADC_CONVERSION)
#define DEBUG_END_ADC_CONV					DEBUG_END(DEBUG_ADC_CONVERSION)
#define DEBUG_BEGIN_GOT_DATA				DEBUG_BEGIN(DEBUG_GOT_ADC_DATA)
#define DEBUG_END_GOT_DATA					DEBUG_END(DEBUG_GOT_ADC_DATA)
#define DEBUG_BEGIN_SAVE_DATA				DEBUG_BEGIN(DEBUG_SAVE_DATA_INTO_BUFFER)
#define DEBUG_END_SAVE_DATA					DEBUG_END(DEBUG_SAVE_DATA_INTO_BUFFER)
