/*

					Oscilloscope AVR

			Copyrights 2008,  Yoshi Hiroshi

*/


#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>PIND 2 16

#include "AdcTiming.h"

/*
		Command Processing
*/
unsigned char GetNumADCs(unsigned char cmnd);
unsigned long GetSamplePeriod(unsigned char cmnd);
unsigned char GetPattern(unsigned char pat);


/*
		Working Strage
*/
volatile unsigned char isRunning;
volatile unsigned long samplePeriod;
unsigned char patternSA[] = { 0x01, 0x02, 0x04, 0x08, 0x10, 0x20, 0x40, 0x80 };
#define nextPatternSA			GPIOR0
unsigned char nextADC;			// next start adc number
#define timingInterruptFlag	GPIOR1
volatile unsigned char numADCs;
volatile unsigned char fasterFlag;

/*
=======================================================================
						StopTimer
=======================================================================
*/
void StopTimer()
{
	TIMSK &= ~(PERIOD_INTERRUPT_MASK | _BV(ICIE1));
	TCCR1B = 0;
}

/*
=======================================================================
						SetTimer
=======================================================================
*/
unsigned char SetTimer(unsigned long period)
{
	nextADC = 0;
	nextPatternSA = patternSA[nextADC];
	timingInterruptFlag = 0;
	TCCR0B = 0;
	TCCR1B = 0;
	if(period < 5)
		return 0;

	isRunning = 1;
	if(samplePeriod < 100)
	{
		fasterFlag = 1;
	} else {
		fasterFlag = 0;
		OCR0A = (30 * 16) / 8 - 1;		// 30uS
		TCCR0A = _BV(WGM01);
		TCCR0B = 0;
	}
	if(period <= 10000)
	{
		OCR1A = (period * 16 / 8) - 1;
		GTCCR = _BV(PSR10);
		TCCR1B = _BV(WGM12) | PERIOD_TIMER_FAST_CLOCK;
		TCNT1 = OCR1A - 4;
		TIMSK &= ~_BV(ICIE1);
	} else {						// 20mS, 50mS, 100mS, 200mS, 500mS
		OCR1A = period / 16 /* * 16 / 256 */ - 1;
		TCNT1 = OCR1A - 1;
		GTCCR = _BV(PSR10);
		TCCR1B = _BV(WGM12) | PERIOD_TIMER_SLOW_CLOCK;
		TIMSK |= _BV(ICIE1);
	}
	TIMSK |= PERIOD_INTERRUPT_MASK;
	return 1;
}

/*
=======================================================================
						TIMER0_COMPA_vect
=======================================================================
*/
ISR(TIMER0_COMPA_vect)
{
	TCCR0B = 0;
	TIMSK &= ~_BV(OCIE0A);
	START_ADC_PORT = 0;
}

/*
=======================================================================
						TIMER1_CAPT1_vect
=======================================================================
*/
ISR(TIMER1_CAPT_vect)
{
	StopTimer();
	isRunning = 0;
}

/*
=======================================================================
						TIMER1_COMPA_vect
=======================================================================
*/
ISR(TIMER1_COMPA_vect, ISR_NAKED)
{
	asm volatile("push r24");
	START_ADC_PORT = nextPatternSA;
	timingInterruptFlag |= 1;
	asm volatile("pop r24");
	reti();
}

/*
=======================================================================
						fastestTrigger		2uS
=======================================================================
*/
void fastestTrigger()
{
	volatile char dummyWait;
	isRunning = 1;
	
	while(1)
	{
		START_ADC_PORT =nextPatternSA ;
		nextADC++;
		if(nextADC >= numADCs)
		{
			nextADC = 0;
			if((CONTROL_PIN & CONTROL_DO_COMMAND) == 0)
			{
				break;
			}
		} else {
			asm volatile("nop");
			asm volatile("nop");
			asm volatile("nop");
			asm volatile("nop");
		}
		nextPatternSA = patternSA[nextADC];
		START_ADC_PORT = 0;
		// Wait for 10 cycles
		{
			dummyWait = 0x12;
			dummyWait++;
			dummyWait = 0;
		}
	}
	isRunning = 0;
}

/*
=======================================================================
						main
=======================================================================
*/
int main()
{
	unsigned char controlPort;
	START_ADC_PORT = 0;
	START_ADC_DDR = 0xFF;
	CONTROL_DDR = 0x04;
	CONTROL_PORT = 0x7B;
	TCCR1A = 0;
	numADCs = 1;
	isRunning = 0;
	sei();

	while(1)
	{
		controlPort = CONTROL_PIN;
		if((controlPort & CONTROL_DO_COMMAND) != 0)
		{
			switch((controlPort & CONTROL_COMMAND) >> 3)
			{
			case 4:						// SET numADCs
				numADCs = GetNumADCs(controlPort);
				while((CONTROL_PIN & CONTROL_DO_COMMAND) != 0);
				break;
			case 5:						// SINGLE
				START_ADC_PORT = GetPattern(controlPort);
				for(volatile char wait1 = 0; wait1 < 1; wait1++);
				START_ADC_PORT = 0;
				while((CONTROL_PIN & CONTROL_DO_COMMAND) != 0);
				break;
			default:
				samplePeriod = GetSamplePeriod(controlPort);
				if(SetTimer(samplePeriod) != 0)		// start timer for 5uS - 500mS period
				{
					while(isRunning != 0)
					{
						cli();
						sleep_enable();
						sei();
						sleep_cpu();
						cli();
						sleep_disable();
						sei();
						if(timingInterruptFlag)
						{
							timingInterruptFlag = 0;
							if(fasterFlag)
							{
								nextADC++;
								if(nextADC >= numADCs) 
								{
									nextADC = 0;
									if((CONTROL_PIN & CONTROL_DO_COMMAND) == 0)
									{
										StopTimer();
										isRunning = 0;
									}
								}
								nextPatternSA = patternSA[nextADC];
								START_ADC_PORT = 0;
							} else {
								if((CONTROL_PIN & CONTROL_DO_COMMAND) == 0)
								{
									StopTimer();
									isRunning = 0;
								}
								TCNT0 = 0;
								TCCR0B = _BV(CS01);
								TIMSK |= _BV(OCIE0A);
							}
						}
					}
				} else {
					fastestTrigger();			// 2uS period
				}
				break;
			}
		} else {
			START_ADC_PORT = 0;
		}
	} // while
}
