/*

					Oscilloscope AVR

			Copyrights 2008,  Yoshi Hiroshi

*/


#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/sleep.h>

#include "AdcTiming.h"

/*
=======================================================================
						GetNumADCs
=======================================================================
*/
unsigned char GetNumADCs(unsigned char cmnd)
{
	unsigned char num = 1;
	switch(cmnd & CONTROL_PARAM)
	{
		case 0:
			num = 1;
			break;
		case _BV(PARAM_0):
			num = 2;
			break;
		case _BV(PARAM_1):
			num = 4;
			break;
		default:
			num = 8;
			break;
	}
	return num;
}

/*
=======================================================================
						GetSamplePeriod
=======================================================================
*/
unsigned long GetSamplePeriod(unsigned char cmnd)
{
	unsigned long period = 2;
	unsigned long multiplier = 1;
	switch(cmnd & CONTROL_PARAM)
	{
		case _BV(PARAM_0):
			period = 1;
			break;
		case _BV(PARAM_1):
			period = 2;
			break;
		case _BV(PARAM_1) | _BV(PARAM_0):
			period = 5;
			break;
	}
	switch((cmnd & CONTROL_COMMAND) >> 3)
	{
		case 6:									// 6: x  1uS
			multiplier = 1;
			break;
		case 7:									// 7: x 10uS
			multiplier = 10;
			break;
		case 0:									// 0/8: x100uS
			multiplier = 100;
			break;
		case 1:									// 1/9: x  1mS
			multiplier = 1000;
			break;
		case 2:									// 2/A: x 10mS
			multiplier = 10000;
			break;
		case 3:									// 3/B: x 100mS
			multiplier = 100000;
			break;
	}
	period *= multiplier;
	return period;
}

/*
=======================================================================
						GetPattern
=======================================================================
*/
unsigned char GetPattern(unsigned char pat)
{
	unsigned char pattern;
	pat &= CONTROL_PARAM;
	if(pat == 0)					// 1 AVRs
	{
		pattern = 0x01;
	} 
	else if(pat == _BV(PARAM_0)) 	// 2 AVRs
	{
		pattern = 0x03;
	} 
	else if(pat == _BV(PARAM_1)) 	// 4 AVRs
	{
		pattern = 0x0F;
	} 
	else 							// 8 AVRs
	{
		pattern = 0xFF;
	}
	return pattern;
}

