/*

					Oscilloscope AVR

			Copyrights 2008,  Yoshi Hiroshi

		Control Block - ATmega644P/ATmega324P

*/

#include <avr/io.h>

#include "OscilloCtrl.h"
#include "Sampling.h"
#include "AdcManager.h"
#include "Spi.h"

#define ADC_SLOW_WAIT					11
#define ADC_FAST_WAIT					6
#define ADC_WAIT_FACTOR					ADC_SLOW_WAIT
#define WAIT_FOR_ADC_COMPLETE(n)		for(volatile int waitCount1 = 10 * (n); waitCount1 >= 0; waitCount1--);

unsigned char numChannels;
unsigned char numADCs;
unsigned int buffSize;
unsigned char slowestPeriodCode;
unsigned char fastestPeriodCode;


//**********************************************************
//						GetNumChannels
//**********************************************************
unsigned char GetNumChannels()
{
	return numChannels;
}

//**********************************************************
//						GetNumADCs
//**********************************************************
unsigned char GetNumADCs()
{
	return numADCs;
}

//**********************************************************
//						GetBuffSize
//**********************************************************
unsigned int GetBuffSize()
{
	return buffSize;
}


//**********************************************************
//						GetVref
//**********************************************************
unsigned int GetVref()
{
	return ADC_VREF_VALUE;
}

//**********************************************************
//						GetSlowestPeriod
//**********************************************************
unsigned long GetSlowestPeriodCode()
{
	return slowestPeriodCode;
}

//**********************************************************
//						GetarstestPeriod
//**********************************************************
unsigned long GetFastestPeriodCode()
{
	return fastestPeriodCode;
}

//**********************************************************
//						GetDeviceType
//**********************************************************
unsigned char GetDeviceType(unsigned char ch, unsigned char unit)
{
	SPITransmit(ch, unit, ADC_COMMAND_CODE_GET_DEVICE, SPI_TRANSIMIT_START);
	WAIT_FOR_SPI_READY(2);
	unsigned char s = SPITransmit(ch, unit, ADC_COMMAND_CODE_NOP, SPI_TRANSIMIT_TERMINATE);
	return s;
}

//**********************************************************
//						ClearAdcAll
//**********************************************************
void ClearAdcAll(unsigned char ch, unsigned char unit)
{
	SPITransmit(ch, unit, ADC_COMMAND_CODE_CLEARALL, SPI_TRANSIMIT_TERMINATE);
}

//**********************************************************
//						SetAdcRange
//**********************************************************
void SetAdcRange(unsigned char ch, unsigned char val)
{
	SPITransmit(ch, 0, ADC_COMMAND_CODE_SET_CHANNEL_RANGE, SPI_TRANSIMIT_TERMINATE);
	SPITransmit(ch, 1, ADC_COMMAND_CODE_SET_CHANNEL_RANGE, SPI_TRANSIMIT_TERMINATE);
	SPITransmit(ch, 2, ADC_COMMAND_CODE_SET_CHANNEL_RANGE, SPI_TRANSIMIT_TERMINATE);
	SPITransmit(ch, 3, ADC_COMMAND_CODE_SET_CHANNEL_RANGE, SPI_TRANSIMIT_TERMINATE);
	SPITransmit(ch, 4, ADC_COMMAND_CODE_SET_CHANNEL_RANGE, SPI_TRANSIMIT_TERMINATE);
	SPITransmit(ch, 5, ADC_COMMAND_CODE_SET_CHANNEL_RANGE, SPI_TRANSIMIT_TERMINATE);
	SPITransmit(ch, 6, ADC_COMMAND_CODE_SET_CHANNEL_RANGE, SPI_TRANSIMIT_TERMINATE);
	SPITransmit(ch, 7, ADC_COMMAND_CODE_SET_CHANNEL_RANGE, SPI_TRANSIMIT_TERMINATE);

	SPITransmit(ch, 0, val, SPI_TRANSIMIT_TERMINATE);
	SPITransmit(ch, 1, val, SPI_TRANSIMIT_TERMINATE);
	SPITransmit(ch, 2, val, SPI_TRANSIMIT_TERMINATE);
	SPITransmit(ch, 3, val, SPI_TRANSIMIT_TERMINATE);
	SPITransmit(ch, 4, val, SPI_TRANSIMIT_TERMINATE);
	SPITransmit(ch, 5, val, SPI_TRANSIMIT_TERMINATE);
	SPITransmit(ch, 6, val, SPI_TRANSIMIT_TERMINATE);
	SPITransmit(ch, 7, val, SPI_TRANSIMIT_TERMINATE);
}

//**********************************************************
//						SetAdcPosition
//**********************************************************
void SetAdcPosition(unsigned char ch, unsigned char val)
{
	SPITransmit(ch, 0, ADC_COMMAND_CODE_SET_CHANNEL_POSITION, SPI_TRANSIMIT_START);
	WAIT_FOR_SPI_READY(2);
	SPITransmit(ch, 0, val, SPI_TRANSIMIT_TERMINATE);
}

//**********************************************************
//						SetAdcTriggerMode
//**********************************************************
void SetAdcTriggerMode(unsigned char ch, unsigned char val)
{
	SPITransmit(ch, 0, ADC_COMMAND_CODE_SET_CHANNEL_TRG_MODE, SPI_TRANSIMIT_TERMINATE);
	SPITransmit(ch, 1, ADC_COMMAND_CODE_SET_CHANNEL_TRG_MODE, SPI_TRANSIMIT_TERMINATE);
	SPITransmit(ch, 2, ADC_COMMAND_CODE_SET_CHANNEL_TRG_MODE, SPI_TRANSIMIT_TERMINATE);
	SPITransmit(ch, 3, ADC_COMMAND_CODE_SET_CHANNEL_TRG_MODE, SPI_TRANSIMIT_TERMINATE);
	SPITransmit(ch, 4, ADC_COMMAND_CODE_SET_CHANNEL_TRG_MODE, SPI_TRANSIMIT_TERMINATE);
	SPITransmit(ch, 5, ADC_COMMAND_CODE_SET_CHANNEL_TRG_MODE, SPI_TRANSIMIT_TERMINATE);
	SPITransmit(ch, 6, ADC_COMMAND_CODE_SET_CHANNEL_TRG_MODE, SPI_TRANSIMIT_TERMINATE);
	SPITransmit(ch, 7, ADC_COMMAND_CODE_SET_CHANNEL_TRG_MODE, SPI_TRANSIMIT_TERMINATE);

	SPITransmit(ch, 0, val, SPI_TRANSIMIT_TERMINATE);
	SPITransmit(ch, 1, val, SPI_TRANSIMIT_TERMINATE);
	SPITransmit(ch, 2, val, SPI_TRANSIMIT_TERMINATE);
	SPITransmit(ch, 3, val, SPI_TRANSIMIT_TERMINATE);
	SPITransmit(ch, 4, val, SPI_TRANSIMIT_TERMINATE);
	SPITransmit(ch, 5, val, SPI_TRANSIMIT_TERMINATE);
	SPITransmit(ch, 6, val, SPI_TRANSIMIT_TERMINATE);
	SPITransmit(ch, 7, val, SPI_TRANSIMIT_TERMINATE);
}

//**********************************************************
//						SetAdcTriggerLevel
//**********************************************************
void SetAdcTriggerLevel(unsigned char ch, unsigned char val)
{
	SPITransmit(ch, 0, ADC_COMMAND_CODE_SET_CHANNEL_TRG_LEVEL, SPI_TRANSIMIT_TERMINATE);
	SPITransmit(ch, 1, ADC_COMMAND_CODE_SET_CHANNEL_TRG_LEVEL, SPI_TRANSIMIT_TERMINATE);
	SPITransmit(ch, 2, ADC_COMMAND_CODE_SET_CHANNEL_TRG_LEVEL, SPI_TRANSIMIT_TERMINATE);
	SPITransmit(ch, 3, ADC_COMMAND_CODE_SET_CHANNEL_TRG_LEVEL, SPI_TRANSIMIT_TERMINATE);
	SPITransmit(ch, 4, ADC_COMMAND_CODE_SET_CHANNEL_TRG_LEVEL, SPI_TRANSIMIT_TERMINATE);
	SPITransmit(ch, 5, ADC_COMMAND_CODE_SET_CHANNEL_TRG_LEVEL, SPI_TRANSIMIT_TERMINATE);
	SPITransmit(ch, 6, ADC_COMMAND_CODE_SET_CHANNEL_TRG_LEVEL, SPI_TRANSIMIT_TERMINATE);
	SPITransmit(ch, 7, ADC_COMMAND_CODE_SET_CHANNEL_TRG_LEVEL, SPI_TRANSIMIT_TERMINATE);

	SPITransmit(ch, 0, val, SPI_TRANSIMIT_TERMINATE);
	SPITransmit(ch, 1, val, SPI_TRANSIMIT_TERMINATE);
	SPITransmit(ch, 2, val, SPI_TRANSIMIT_TERMINATE);
	SPITransmit(ch, 3, val, SPI_TRANSIMIT_TERMINATE);
	SPITransmit(ch, 4, val, SPI_TRANSIMIT_TERMINATE);
	SPITransmit(ch, 5, val, SPI_TRANSIMIT_TERMINATE);
	SPITransmit(ch, 6, val, SPI_TRANSIMIT_TERMINATE);
	SPITransmit(ch, 7, val, SPI_TRANSIMIT_TERMINATE);
}

//**********************************************************
//						SetAdcSwitch
//**********************************************************
void SetAdcSwitch(unsigned char ch, unsigned char val)
{
	SPITransmit(ch, 0, ADC_COMMAND_CODE_SET_CHANNEL_SWITCH, SPI_TRANSIMIT_START);
	WAIT_FOR_SPI_READY(2);
	SPITransmit(ch, 0, val, SPI_TRANSIMIT_TERMINATE);
}

//**********************************************************
//						GetAdcNumData
//**********************************************************
unsigned int GetAdcNumData(unsigned char ch, unsigned char unit)
{
	unsigned int result ;
	SPITransmit(ch, unit, ADC_COMMAND_CODE_FETCH_NUM_DATA_H, SPI_TRANSIMIT_START);
	WAIT_FOR_SPI_READY(2);
	result = SPITransmit(ch, unit, ADC_COMMAND_CODE_FETCH_NUM_DATA_L, SPI_TRANSIMIT_START);
	WAIT_FOR_SPI_READY(2);
	result = (result << 8) | SPITransmit(ch, unit, ADC_COMMAND_CODE_NOP, SPI_TRANSIMIT_TERMINATE);
	return result;
}

//**********************************************************
//						GetAdcFirstData
//**********************************************************
unsigned char GetAdcFirstData(unsigned char ch, unsigned char unit)
{
	SPITransmit(ch, unit, ADC_COMMAND_CODE_FETCH_FIRST, SPI_TRANSIMIT_START);
	WAIT_FOR_SPI_READY(2);
	return SPITransmit(ch, unit, ADC_COMMAND_CODE_NOP, SPI_TRANSIMIT_TERMINATE);
}

//**********************************************************
//						GetAdcNextData
//**********************************************************
unsigned char GetAdcNextData(unsigned char ch, unsigned char unit)
{
	SPITransmit(ch, unit, ADC_COMMAND_CODE_FETCH_NEXT, SPI_TRANSIMIT_START);
	WAIT_FOR_SPI_READY(2);
	return SPITransmit(ch, unit, ADC_COMMAND_CODE_NOP, SPI_TRANSIMIT_TERMINATE);
}


//**********************************************************
//				check for ADC configuration
//**********************************************************
void CheckConfiguration()
{
	unsigned char device;
	numChannels = 2;
	numADCs = 8;
	slowestPeriodCode = TIMEBASE_CODE_500mS;		// 500mS = 500,000uS
	buffSize = DEVICE_BUFFER_SIZE_4K;				// SRAM = 4KB

	for(volatile char chCount = 0; chCount < numChannels; chCount++)
	{
		for(volatile char unitCount = 0; unitCount < 8; unitCount++)
		{
			device = GetDeviceType(chCount, unitCount);
			switch(device)
			{
				case DEVICE_TYPE_ATmega88:
				case DEVICE_TYPE_ATmega168:
				case DEVICE_TYPE_ATmega164P:
					if(buffSize > DEVICE_BUFFER_SIZE_1K)
					{
						buffSize = DEVICE_BUFFER_SIZE_1K;
					}
					break;
				case DEVICE_TYPE_ATmega324P:
					if(buffSize > DEVICE_BUFFER_SIZE_2K)
					{
						buffSize = DEVICE_BUFFER_SIZE_2K;
					}
					break;
				case DEVICE_TYPE_ATmega644P:
					if(buffSize > DEVICE_BUFFER_SIZE_4K)
					{
						buffSize = DEVICE_BUFFER_SIZE_4K;
					}
					break;
				default:
					if(numADCs > unitCount)
					{
						if(unitCount != 0)
						{
							numADCs = unitCount;
						} else {
							unitCount = 8;	// If ADC Block Not exist, Skip Check
						}
					}
					break;
			}
		}
	}

	if((DEBUG_PIN & _BV(DEBUG_BIT)) == 0)
	{
		numADCs = 8;
	}

	switch(numADCs)
	{
		case 1:
			fastestPeriodCode = TIMEBASE_CODE_20uS;
			numADCs = 1;
			break;
		case 2:
		case 3:
			fastestPeriodCode = TIMEBASE_CODE_10uS;
			numADCs = 2;
			break;
		case 4:
		case 5:
		case 6:
		case 7:
			fastestPeriodCode = TIMEBASE_CODE_5uS;
			numADCs = 4;
			break;
		case 8:
			fastestPeriodCode = TIMEBASE_CODE_2uS;
			break;
		default:
			fastestPeriodCode = TIMEBASE_CODE_100uS;
			break;
	}
}

