/*

					Oscilloscope AVR

			Copyrights 2008,  Yoshi Hiroshi

		Control Block - ATmega644P/ATmega324P

*/

#if !defined(__ADC_MANAGER_HEADER_INCLUDED__)
#define __ADC_MANAGER_HEADER_INCLUDED__

#define ADC_VREF_VALUE						2500

#define DEVICE_BUFFER_SIZE_1K				(120*7)
#define DEVICE_BUFFER_SIZE_2K				(120*15)
#define DEVICE_BUFFER_SIZE_4K				(120*32)

extern unsigned char numChannels;
extern unsigned char numADCs;
extern unsigned int buffSize;

void CheckConfiguration();

unsigned char GetNumChannels();
unsigned char GetNumADCs();
unsigned int GetBuffSize();
unsigned int GetVref();
unsigned long GetSlowestPeriodCode();
unsigned long GetFastestPeriodCode();
unsigned char GetAdcData(unsigned char ch, unsigned char unit);

unsigned char GetDeviceType(unsigned char ch, unsigned char unit);
void ClearAdcAll(unsigned char ch, unsigned char unit);

void SetAdcRange(unsigned char ch, unsigned char val);
void SetAdcPosition(unsigned char ch, unsigned char val);
void SetAdcTriggerMode(unsigned char ch, unsigned char val);
void SetAdcTriggerLevel(unsigned char ch, unsigned char val);
void SetAdcSwitch(unsigned char ch, unsigned char val);

unsigned int GetAdcNumData(unsigned char ch, unsigned char unit);
unsigned char GetAdcFirstData(unsigned char ch, unsigned char unit);
unsigned char GetAdcNextData(unsigned char ch, unsigned char unit);


//#define AUTOTRIGGER_TIMEOUT				1000000	// 1000mS
//#define AUTOTRIGGER_INFINITE			10000000	// 10S

//
//		ADC Command Macro
//
#define ADC_CLEAR_CH1(unit)			SPITransmit_CH1(unit, ADC_COMMAND_CODE_CLEARALL, SPI_TRANSIMIT_TERMINATE)
#define ADC_CLEAR_CH2(unit)			SPITransmit_CH2(unit, ADC_COMMAND_CODE_CLEARALL, SPI_TRANSIMIT_TERMINATE)
#define ADC_CLEAR(unit)				ADC_CLEAR_CH1(unit);ADC_CLEAR_CH2(unit)
#define ADC_CLEAR_ALL				ADC_CLEAR(0);ADC_CLEAR(1);ADC_CLEAR(2);ADC_CLEAR(3);ADC_CLEAR(4);ADC_CLEAR(5);ADC_CLEAR(6);ADC_CLEAR(7)
//
#define ADC_GET_VALUE_CH1(unit)		SPITransmit_CH1(unit, ADC_COMMAND_CODE_GET_CHANNEL_VALUE, SPI_TRANSIMIT_TERMINATE)
#define ADC_GET_VALUE_CH2(unit)		SPITransmit_CH2(unit, ADC_COMMAND_CODE_GET_CHANNEL_VALUE, SPI_TRANSIMIT_TERMINATE)
#define ADC_GET_VALUE(unit)			ADC_GET_VALUE_CH1(unit);ADC_GET_VALUE_CH2(unit)
#define ADC_GET_VALUE_ALL			ADC_GET_VALUE(0);ADC_GET_VALUE(1);ADC_GET_VALUE(2);ADC_GET_VALUE(3);ADC_GET_VALUE(4);ADC_GET_VALUE(5);ADC_GET_VALUE(6);ADC_GET_VALUE(7)
//
#define ADC_FETCH_FIRST_CH1(unit)	SPITransmit_CH1(unit, ADC_COMMAND_CODE_FETCH_FIRST, SPI_TRANSIMIT_TERMINATE)
#define ADC_FETCH_FIRST_CH2(unit)	SPITransmit_CH2(unit, ADC_COMMAND_CODE_FETCH_FIRST, SPI_TRANSIMIT_TERMINATE)
#define ADC_FETCH_FIRST(unit)		ADC_FETCH_FIRST_CH1(unit);ADC_FETCH_FIRST_CH2(unit)
#define ADC_FETCH_FIRST_ALL			ADC_FETCH_FIRST(0);ADC_FETCH_FIRST(1);ADC_FETCH_FIRST(2);ADC_FETCH_FIRST(3);ADC_FETCH_FIRST(4);ADC_FETCH_FIRST(5);ADC_FETCH_FIRST(6);ADC_FETCH_FIRST(7)
//
#define ADC_FETCH_NEXT_CH1(unit)	SPITransmit_CH1(unit, ADC_COMMAND_CODE_FETCH_NEXT, SPI_TRANSIMIT_TERMINATE)
#define ADC_FETCH_NEXT_CH2(unit)	SPITransmit_CH2(unit, ADC_COMMAND_CODE_FETCH_NEXT, SPI_TRANSIMIT_TERMINATE)
#define ADC_FETCH_NEXT(unit)		ADC_FETCH_NEXT_CH1(unit);ADC_FETCH_NEXT_CH2(unit)
#define ADC_FETCH_NEXT_ALL			ADC_FETCH_NEXT(0);ADC_FETCH_NEXT(1);ADC_FETCH_NEXT(2);ADC_FETCH_NEXT(3);ADC_FETCH_NEXT(4);ADC_FETCH_NEXT(5);ADC_FETCH_NEXT(6);ADC_FETCH_NEXT(7)


#endif
