/*

					Oscilloscope AVR

			Copyrights 2008,  Yoshi Hiroshi

		Control Block - ATmega644P/ATmega324P
*/

#include <util/atomic.h>
#include "Buffer.h"

rxBuffType rxBuff[NUM_RECIEVE_BUFFERS];
rxBuffType* freeRxBuffRoot;
rxBuffType* freeRxBuffLast;
rxBuffType* usedRxBuffRoot;
rxBuffType* usedRxBuffLast;

txBuffType txBuff[NUM_TRANSMIT_BUFFERS];
txBuffType* freeTxBuffRoot;
txBuffType* freeTxBuffLast;
txBuffType* usedTxBuffRoot;
txBuffType* usedTxBuffLast;

//==========================================================
//		Buffer Manageent
//==========================================================

void InitBuff()
{
	for(int indexRx = 0; indexRx < NUM_RECIEVE_BUFFERS; indexRx++)
	{
		if(indexRx < (NUM_RECIEVE_BUFFERS - 1))
		{
			rxBuff[indexRx].next = &rxBuff[indexRx + 1];
		} else {
			rxBuff[indexRx].next = 0;
		}
	}
	freeRxBuffRoot	= &rxBuff[0];
	freeRxBuffLast	= &rxBuff[NUM_RECIEVE_BUFFERS - 1];
	usedRxBuffRoot	= 0;
	usedRxBuffLast	= 0;

	for(int indexTx = 0; indexTx < NUM_TRANSMIT_BUFFERS; indexTx++)
	{
		if(indexTx < (NUM_TRANSMIT_BUFFERS - 1))
		{
			txBuff[indexTx].next = &txBuff[indexTx + 1];
		} else {
			txBuff[indexTx].next = 0;
		}
	}
	freeTxBuffRoot	= &txBuff[0];
	freeTxBuffLast	= &txBuff[NUM_TRANSMIT_BUFFERS - 1];
	usedTxBuffRoot	= 0;
	usedTxBuffLast	= 0;
}

//**********************************************************
//		Get Free Recieve Buffer
//**********************************************************
rxBuffType* GetFreeRxBuff(unsigned char wait)
{
	rxBuffType* result;
	if(wait != 0) while(freeRxBuffRoot == 0);	// No Free Buffer. Wait for release buffer
	ATOMIC_BLOCK(ATOMIC_RESTORESTATE)
	{
		result = freeRxBuffRoot;
		if(result != 0)
		{
			freeRxBuffRoot = result->next;
			if(freeRxBuffRoot == 0)
			{
				freeRxBuffLast = 0;
			}
		}
	}
	return result;
}

//**********************************************************
//		Put Free Recieve Buffer
//**********************************************************
void PutFreeRxBuff(rxBuffType* ptr)
{
	ATOMIC_BLOCK(ATOMIC_RESTORESTATE)
	{
		ptr->next = 0;
		if(freeRxBuffLast != 0)
		{
			freeRxBuffLast->next = ptr;
		} else {
			freeRxBuffRoot = ptr;
		}
		freeRxBuffLast = ptr;
	}
}

//**********************************************************
//		Is Free Recieve Buffer Empty
//**********************************************************
//unsigned char IsFreeRxBuffEmpty()
//{
//	return (freeRxBuffRoot == 0);
//}

//**********************************************************
//		Get Used Recieved Buffer (Recieved Data Inside)
//**********************************************************
rxBuffType* GetUsedRxBuff(unsigned char wait)
{
	rxBuffType* result;
	if(wait != 0) while(usedRxBuffRoot == 0);	// No Used Buffer. Wait for release buffer
	ATOMIC_BLOCK(ATOMIC_RESTORESTATE)
	{
		result = usedRxBuffRoot;
		if(result != 0)
		{
			usedRxBuffRoot = result->next;
			if(usedRxBuffRoot == 0)
			{
				usedRxBuffLast = 0;
			}
		}
	}
	return result;
}

//**********************************************************
//		Put Used Recieved Buffer
//**********************************************************
void PutUsedRxBuff(rxBuffType* ptr)
{
	ATOMIC_BLOCK(ATOMIC_RESTORESTATE)
	{
		ptr->next = 0;
		if(usedRxBuffLast != 0)
		{
			usedRxBuffLast->next = ptr;
		} else {
			usedRxBuffRoot = ptr;
		}
		usedRxBuffLast = ptr;
	}
}

//**********************************************************
//		Is Used Recieve Buffer Empty
//**********************************************************
unsigned char IsUsedRxBuffEmpty()
{
	return (usedRxBuffRoot == 0);
}

//**********************************************************
//		Get Free Transmit Buffer
//**********************************************************
txBuffType* GetFreeTxBuff(unsigned char wait)
{
	txBuffType* result = 0;
	if(wait != 0) while(freeTxBuffRoot == 0);	// No Free Buffer. Wait for release buffer
	ATOMIC_BLOCK(ATOMIC_RESTORESTATE)
	{
		result = freeTxBuffRoot;
		if(result != 0)
		{
			freeTxBuffRoot = result->next;
			if(freeTxBuffRoot == 0)
			{
				freeTxBuffLast = 0;
			}
		}
	}
	return result;
}

//**********************************************************
//		Put Free Transmit Buffer
//**********************************************************
void PutFreeTxBuff(txBuffType* ptr)
{
	ATOMIC_BLOCK(ATOMIC_RESTORESTATE)
	{
		ptr->next = 0;
		if(freeTxBuffLast != 0)
		{
			freeTxBuffLast->next = ptr;
		} else {
			freeTxBuffRoot = ptr;
		}
		freeTxBuffLast = ptr;
	}
}

//**********************************************************
//		Is Free Transmit Buffer Empty
//**********************************************************
//unsigned char IsFreeTxBuffEmpty()
//{
//	return (freeTxBuffRoot == 0);
//}

//**********************************************************
//		Get Used Transmit Buffer
//**********************************************************
txBuffType* GetUsedTxBuff(unsigned char wait)
{
	txBuffType* result = 0;
	if(wait != 0) while(usedTxBuffRoot == 0);	// No Free Buffer. Wait for release buffer
	ATOMIC_BLOCK(ATOMIC_RESTORESTATE)
	{
		result = usedTxBuffRoot;
		if(result != 0)
		{
			usedTxBuffRoot = result->next;
			if(usedTxBuffRoot == 0)
			{
				usedTxBuffLast = 0;
			}
		}
	}
	return result;
}

//**********************************************************
//		Put Used Transmit Buffer
//**********************************************************
void PutUsedTxBuff(txBuffType* ptr)
{
	ATOMIC_BLOCK(ATOMIC_RESTORESTATE)
	{
		ptr->next = 0;
		if(usedTxBuffLast != 0)
		{
			usedTxBuffLast->next = ptr;
		} else {
			usedTxBuffRoot = ptr;
		}
		usedTxBuffLast = ptr;
	}
}

//**********************************************************
//		Is Used Transmit Buffer Empty
//**********************************************************
//unsigned char IsUsedTxBuffEmpty()
//{
//	return (usedTxBuffRoot == 0);
//}
