/*

					Oscilloscope AVR

			Copyrights 2008,  Yoshi Hiroshi

		Control Block - ATmega644P/ATmega324P
*/

#include "OscilloCtrl.h"

#if defined(__AVR_CTRL_INCLUDED__)

void MakeReplyBadCommand(unsigned char* buff, unsigned char cmnd)
{
	SET_CMND(buff, 2);
	SET_CMND(buff, (cmnd | COMMAND_CODE_REPLY));
	SET_CMND(buff, 0xFF);
}

void MakeReplyConfiguration(unsigned char* buff, unsigned char numChannels, unsigned char numADCs, unsigned int Vref, unsigned char slowestPeriodCode, unsigned char fastestPeriodCode, unsigned int buffSize)
{
	SET_CMND(buff, 9);
	SET_CMND(buff, (COMMAND_CODE_GET_CONFIGURATION | COMMAND_CODE_REPLY));
	SET_CMND(buff, numChannels);
	SET_CMND(buff, numADCs);
	SET_CMND(buff, ((unsigned char)((Vref >> 8) & 0xFF)));				// mV
	SET_CMND(buff, ((unsigned char)(Vref & 0xFF)));
	SET_CMND(buff, slowestPeriodCode);
	SET_CMND(buff, fastestPeriodCode);
	SET_CMND(buff, ((unsigned char)((buffSize >> 8) & 0xFF)));
	SET_CMND(buff, ((unsigned char)(buffSize & 0xFF)));
}

void MakeReplySettings(unsigned char* buff, unsigned char range1, unsigned char pos1, unsigned char sw1, unsigned char range2, unsigned char pos2, unsigned char sw2, unsigned char level, unsigned char val1, unsigned char val2)
{
	SET_CMND(buff, 10);
	SET_CMND(buff, (COMMAND_CODE_GET_SETTINGS | COMMAND_CODE_REPLY));
	SET_CMND(buff, range1);
	SET_CMND(buff, pos1);
	SET_CMND(buff, sw1);
	SET_CMND(buff, range2);
	SET_CMND(buff, pos2);
	SET_CMND(buff, sw2);
	SET_CMND(buff, level);
	SET_CMND(buff, val1);
	SET_CMND(buff, val2);
}

void MakeReplySetSettings(unsigned char* buff, unsigned char result)
{
	SET_CMND(buff, 2);
	SET_CMND(buff, (COMMAND_CODE_SET_SETTINGS | COMMAND_CODE_REPLY));
	SET_CMND(buff, result);
}

void MakeReplySampling(unsigned char* buff, unsigned char channel, long seq, unsigned char numData, unsigned char* dataBuff)
{
	SET_CMND(buff, ((unsigned char)(5 + numData)));
	SET_CMND(buff, (COMMAND_CODE_START_SAMPLING | COMMAND_CODE_REPLY));
	SET_CMND(buff, channel);
	SET_CMND(buff, ((unsigned char)((seq >> 16) & 0xFF)));
	SET_CMND(buff, ((unsigned char)((seq >> 8) & 0xFF)));
	SET_CMND(buff, ((unsigned char)(seq & 0xFF)));
	for(unsigned int index = 0; index < numData; index++)
		SET_CMND(buff, *(dataBuff++));
}

void MakeReplyStopSampling(unsigned char* buff, unsigned int numData)
{
	SET_CMND(buff, 3);
	SET_CMND(buff, (COMMAND_CODE_STOP_SAMPLING | COMMAND_CODE_REPLY));
	SET_CMND(buff, ((unsigned char)((numData >> 8) & 0xFF)));
	SET_CMND(buff, ((unsigned char)(numData & 0xFF)));
}

void MakeReplyData(unsigned char* buff, unsigned char channel, unsigned char unit, unsigned char data)
{
	SET_CMND(buff, 4);
	SET_CMND(buff, (COMMAND_CODE_GET_DATA | COMMAND_CODE_REPLY));
	SET_CMND(buff, channel);
	SET_CMND(buff, unit);
	SET_CMND(buff, data);
}

void MakeReplyRange(unsigned char* buff, unsigned char channel, unsigned char unit, unsigned char range, unsigned char val)
{
	SET_CMND(buff, 5);
	SET_CMND(buff, (COMMAND_CODE_GET_RANGE | COMMAND_CODE_REPLY));
	SET_CMND(buff, channel);
	SET_CMND(buff, unit);
	SET_CMND(buff, range);
	SET_CMND(buff, val);
}

void MakeReplyPosition(unsigned char* buff, unsigned char channel, unsigned char unit, unsigned char pos)
{
	SET_CMND(buff, 4);
	SET_CMND(buff, (COMMAND_CODE_GET_POSITION | COMMAND_CODE_REPLY));
	SET_CMND(buff, channel);
	SET_CMND(buff, unit);
	SET_CMND(buff, pos);
}

void MakeReplyTriggerLevel(unsigned char* buff, unsigned char channel, unsigned char unit, unsigned char level)
{
	SET_CMND(buff, 4);
	SET_CMND(buff, (COMMAND_CODE_GET_TRIGGER_LEVEL | COMMAND_CODE_REPLY));
	SET_CMND(buff, channel);
	SET_CMND(buff, unit);
	SET_CMND(buff, level);
}

void MakeReplySwitch(unsigned char* buff, unsigned char channel, unsigned char unit, unsigned char s)
{
	SET_CMND(buff, 4);
	SET_CMND(buff, (COMMAND_CODE_GET_SWITCH | COMMAND_CODE_REPLY));
	SET_CMND(buff, channel);
	SET_CMND(buff, unit);
	SET_CMND(buff, s);
}

void MakeReplyDCAC(unsigned char* buff, unsigned char channel, unsigned char unit, unsigned char flag)
{
	SET_CMND(buff, 4);
	SET_CMND(buff, (COMMAND_CODE_GET_DCAC | COMMAND_CODE_REPLY));
	SET_CMND(buff, channel);
	SET_CMND(buff, unit);
	SET_CMND(buff, flag);									// 0:AC, 1:DC
}

void MakeReplySIGGND(unsigned char* buff, unsigned char channel, unsigned char unit, unsigned char flag)
{
	SET_CMND(buff, 4);
	SET_CMND(buff, (COMMAND_CODE_GET_SIGGND | COMMAND_CODE_REPLY));
	SET_CMND(buff, channel);
	SET_CMND(buff, unit);
	SET_CMND(buff, flag);									// 0:GND, 1:SIGNAL
}


void MakeReplySetRange(unsigned char* buff, unsigned char channel, unsigned char val)
{
	SET_CMND(buff, 3);
	SET_CMND(buff, (COMMAND_CODE_SET_RANGE | COMMAND_CODE_REPLY));
	SET_CMND(buff, channel);
	SET_CMND(buff, val);

}

void MakeReplySetPosition(unsigned char* buff, unsigned char channel, unsigned char val)
{
	SET_CMND(buff, 3);
	SET_CMND(buff, (COMMAND_CODE_SET_POSITION | COMMAND_CODE_REPLY));
	SET_CMND(buff, channel);
	SET_CMND(buff, val);
}

void MakeReplySetTriggerMode(unsigned char* buff, unsigned char channel, unsigned char val)
{
	SET_CMND(buff, 3);
	SET_CMND(buff, (COMMAND_CODE_SET_TRIGGER_MODE | COMMAND_CODE_REPLY));
	SET_CMND(buff, channel);
	SET_CMND(buff, val);
}

void MakeReplySetTriggerLevel(unsigned char* buff, unsigned char channel, unsigned char val)
{
	SET_CMND(buff, 3);
	SET_CMND(buff, (COMMAND_CODE_SET_TRIGGER_LEVEL | COMMAND_CODE_REPLY));
	SET_CMND(buff, channel);
	SET_CMND(buff, val);
}

void MakeReplySetSwitch(unsigned char* buff, unsigned char channel, unsigned char val)
{
	SET_CMND(buff, 3);
	SET_CMND(buff, (COMMAND_CODE_SET_SWITCH | COMMAND_CODE_REPLY));
	SET_CMND(buff, channel);
	SET_CMND(buff, val);
}

void MakeReplyClearAll(unsigned char* buff, unsigned char channel, unsigned char unit)
{
	SET_CMND(buff, 3);
	SET_CMND(buff, (COMMAND_CODE_CLEAR_ALL | COMMAND_CODE_REPLY));
	SET_CMND(buff, channel);
	SET_CMND(buff, unit);
}
/*
void MakeReplyFastMode(unsigned char* buff, unsigned char channel, unsigned char unit)
{
	SET_CMND(buff, 3);
	SET_CMND(buff, (COMMAND_CODE_SET_FAST_MODE | COMMAND_CODE_REPLY));
	SET_CMND(buff, channel);
	SET_CMND(buff, unit);
}

void MakeReplySlowMode(unsigned char* buff, unsigned char channel, unsigned char unit)
{
	SET_CMND(buff, 3);
	SET_CMND(buff, (COMMAND_CODE_SET_SLOW_MODE | COMMAND_CODE_REPLY));
	SET_CMND(buff, channel);
	SET_CMND(buff, unit);
}
*/
void MakeReplyNumData(unsigned char* buff, unsigned char channel, unsigned char unit, unsigned int num)
{
	SET_CMND(buff, 5);
	SET_CMND(buff, (COMMAND_CODE_GET_NUM_DATA | COMMAND_CODE_REPLY));
	SET_CMND(buff, channel);
	SET_CMND(buff, unit);
	SET_CMND(buff, ((unsigned char)((num >> 8) & 0xFF)));				// mV
	SET_CMND(buff, ((unsigned char)(num & 0xFF)));
}

void MakeReplyFirstData(unsigned char* buff, unsigned char channel, unsigned char unit, unsigned char data)
{
	SET_CMND(buff, 4);
	SET_CMND(buff, (COMMAND_CODE_GET_FIRST_DATA | COMMAND_CODE_REPLY));
	SET_CMND(buff, channel);
	SET_CMND(buff, unit);
	SET_CMND(buff, data);
}

void MakeReplyNextData(unsigned char* buff, unsigned char channel, unsigned char unit, unsigned char data)
{
	SET_CMND(buff, 4);
	SET_CMND(buff, (COMMAND_CODE_GET_NEXT_DATA | COMMAND_CODE_REPLY));
	SET_CMND(buff, channel);
	SET_CMND(buff, unit);
	SET_CMND(buff, data);
}

#endif
