/*

					Oscilloscope AVR

			Copyrights 2008,  Yoshi Hiroshi

*/

#ifndef CMNDMSG_H_INCLUDED
#define CMNDMSG_H_INCLUDED

//#define __PC_CTRL_INCLUDED__
//#define __AVR_CTRL_INCLUDED__
//#define __AVR_ADC_INCLUDED__

//===============================================< PC-Controller >=====
#define COMMAND_CODE_UNKNOWN						0x00
#define COMMAND_CODE_REPLY							0x80

#define COMMAND_CODE_RESET_ALL      				0x31
#define COMMAND_CODE_GET_CONFIGURATION				0x32
#define COMMAND_CODE_GET_SETTINGS   				0x33
#define COMMAND_CODE_SET_SETTINGS   				0x34
#define COMMAND_CODE_START_SAMPLING					0x39
#define COMMAND_CODE_STOP_SAMPLING					0x3A

#define COMMAND_CODE_GET_DATA						0x41
#define COMMAND_CODE_GET_RANGE						0x42
#define COMMAND_CODE_GET_POSITION					0x43
#define COMMAND_CODE_GET_TRIGGER_LEVEL				0x44
#define COMMAND_CODE_GET_SWITCH						0x45
#define COMMAND_CODE_GET_DCAC						0x46
#define COMMAND_CODE_GET_SIGGND						0x47

#define COMMAND_CODE_SET_RANGE						0x52		// V2.0
#define COMMAND_CODE_SET_POSITION					0x53		// V2.0
#define COMMAND_CODE_SET_TRIGGER_MODE				0x54		// V2.0
#define COMMAND_CODE_SET_TRIGGER_LEVEL				0x55		// V2.0
#define COMMAND_CODE_SET_SWITCH						0x56		// V2.0

#define COMMAND_CODE_CLEAR_ALL						0x61
#define COMMAND_CODE_SET_FAST_MODE					0x62
#define COMMAND_CODE_SET_SLOW_MODE					0x63
#define COMMAND_CODE_GET_NUM_DATA					0x64
#define COMMAND_CODE_GET_FIRST_DATA					0x65
#define COMMAND_CODE_GET_NEXT_DATA					0x66

#define COMMAND_LEN_RESET_ALL                       12
#define COMMAND_LEN_GET_CONFIGURATION				1
#define COMMAND_LEN_GET_SETTINGS    				1
#define COMMAND_LEN_SET_SETTINGS    				7
#define COMMAND_LEN_START_SAMPLING					10
#define COMMAND_LEN_STOP_SAMPLING					1

#define COMMAND_LEN_GET_DATA						3
#define COMMAND_LEN_GET_RANGE						3
#define COMMAND_LEN_GET_POSITION					3
#define COMMAND_LEN_GET_TRIGGER_MODE				3
#define COMMAND_LEN_GET_TRIGGER_LEVEL				3
#define COMMAND_LEN_GET_SWITCH						3
#define COMMAND_LEN_GET_DCAC						3
#define COMMAND_LEN_GET_SIGGND						3

#define COMMAND_LEN_SET_RANGE						3		// V2.0
#define COMMAND_LEN_SET_POSITION					3		// V2.0
#define COMMAND_LEN_SET_TRIGGER_MODE				3		// V2.0
#define COMMAND_LEN_SET_TRIGGER_LEVEL				3		// V2.0
#define COMMAND_LEN_SET_SWITCH						3		// V2.0

#define COMMAND_LEN_CLEAR_ALL						3
#define COMMAND_LEN_SET_FAST_MODE					3
#define COMMAND_LEN_SET_SLOW_MODE					3
#define COMMAND_LEN_GET_NUM_DATA					3
#define COMMAND_LEN_GET_FIRST_DATA					3
#define COMMAND_LEN_GET_NEXT_DATA					3

//==============================================< Controller-ADC >=====
#define ADC_COMMAND_CODE_NOP						0x20
#define ADC_COMMAND_CODE_GET_DEVICE					0x21

#define ADC_COMMAND_CODE_GET_CHANNEL_VALUE			0x41
#define ADC_COMMAND_CODE_GET_CHANNEL_RANGE			0x42
#define ADC_COMMAND_CODE_GET_CHANNEL_POSITION		0x43
#define ADC_COMMAND_CODE_GET_CHANNEL_TRG_LEVEL		0x44
#define ADC_COMMAND_CODE_GET_CHANNEL_SWITCH			0x45

#define ADC_COMMAND_CODE_SET_CHANNEL_RANGE			0x52		// V2.0
#define ADC_COMMAND_CODE_SET_CHANNEL_POSITION		0x53		// V2.0
#define ADC_COMMAND_CODE_SET_CHANNEL_TRG_MODE		0x54		// V2.0
#define ADC_COMMAND_CODE_SET_CHANNEL_TRG_LEVEL		0x55		// V2.0
#define ADC_COMMAND_CODE_SET_CHANNEL_SWITCH			0x56		// V2.0

#define ADC_COMMAND_CODE_CLEARALL					0x61
#define ADC_COMMAND_CODE_SET_FAST					0x62
#define ADC_COMMAND_CODE_SET_SLOW					0x63
#define ADC_COMMAND_CODE_FETCH_FIRST				0x64
#define ADC_COMMAND_CODE_FETCH_NEXT					0x65
#define ADC_COMMAND_CODE_FETCH_NUM_DATA_H			0x66
#define ADC_COMMAND_CODE_FETCH_NUM_DATA_L			0x67

//-----
#define SET_CMND(cmndPtr, val)						*cmndPtr++ = (val)

//-----
#define DEVICE_TYPE_ATmega88						(0x3A)
#define DEVICE_TYPE_ATmega168						(0x46)
#define DEVICE_TYPE_ATmega164P						(0x4A)
#define DEVICE_TYPE_ATmega324P						(0x58)
#define DEVICE_TYPE_ATmega644P						(0x6A)

//-----
#define CHANNEL_CODE_1								1
#define CHANNEL_CODE_2								2

#define SAMPLEPERIOD_PERIOD_1						0x10
#define SAMPLEPERIOD_PERIOD_2						0x20
#define SAMPLEPERIOD_PERIOD_5						0x50
#define SAMPLEPERIOD_UNIT_1pS						0x00
#define SAMPLEPERIOD_UNIT_10pS						0x01
#define SAMPLEPERIOD_UNIT_100pS						0x02
#define SAMPLEPERIOD_UNIT_1nS						0x03
#define SAMPLEPERIOD_UNIT_10nS						0x04
#define SAMPLEPERIOD_UNIT_100nS						0x05
#define SAMPLEPERIOD_UNIT_1uS						0x06
#define SAMPLEPERIOD_UNIT_10uS						0x07
#define SAMPLEPERIOD_UNIT_100uS						0x08
#define SAMPLEPERIOD_UNIT_1mS						0x09
#define SAMPLEPERIOD_UNIT_10mS						0x0A
#define SAMPLEPERIOD_UNIT_100mS						0x0B

#define TRIGGER_MODE_AUTO							0x80
#define TRIGGER_MODE_NONE							0x00
#define TRIGGER_MODE_PLUS							0x40
#define TRIGGER_MODE_MINUS							0x20
#define TRIGGER_MODE_DELAYED						0x10
#define TRIGGER_MODE_CHANNEL_1						0x01
#define TRIGGER_MODE_CHANNEL_2						0x02

#define DELAY_UNIT_uS								0x02
#define DELAY_UNIT_mS								0x03
#define DELAY_UNIT_S								0x04

#define ADC_SWITCH_DCAC_BIT							0
#define ADC_SWITCH_SIGGND_BIT						1

#if defined(__PC_CTRL_INCLUDED__)
class CmndMsg
{
public:
	static void MakeNullMessage(unsigned char* buff, unsigned char len = 16)
		{
			for(unsigned char index = 0; index < len; index++)
			{
                SET_CMND(buff,0);
			}
		};
	static void MakeResetAll(unsigned char* buff)
		{
			SET_CMND(buff,COMMAND_LEN_RESET_ALL);
			SET_CMND(buff,COMMAND_CODE_RESET_ALL);
            SET_CMND(buff,0);
            SET_CMND(buff,0);
            SET_CMND(buff,0);
            SET_CMND(buff,0);
            SET_CMND(buff,0);
            SET_CMND(buff,0);
            SET_CMND(buff,0);
            SET_CMND(buff,0);
            SET_CMND(buff,0);
			SET_CMND(buff,1);
			SET_CMND(buff,COMMAND_CODE_RESET_ALL);
		};
	static void MakeGetConfiguration(unsigned char* buff)
		{
			SET_CMND(buff,COMMAND_LEN_GET_CONFIGURATION);
			SET_CMND(buff,COMMAND_CODE_GET_CONFIGURATION);
		};
	static void MakeGetSettings(unsigned char* buff)
		{
			SET_CMND(buff,COMMAND_LEN_GET_SETTINGS);
			SET_CMND(buff,COMMAND_CODE_GET_SETTINGS);
		};
	static void MakeSetSettings(unsigned char* buff, unsigned char range1, unsigned char pos1, unsigned char sw1, unsigned char range2, unsigned char pos2, unsigned char sw2)
		{
			SET_CMND(buff,COMMAND_LEN_SET_SETTINGS);
			SET_CMND(buff,COMMAND_CODE_SET_SETTINGS);
			SET_CMND(buff,range1);
			SET_CMND(buff,pos1);
			SET_CMND(buff,sw1);
			SET_CMND(buff,range2);
			SET_CMND(buff,pos2);
			SET_CMND(buff,sw2);
		};
	static void MakeStartSampling(unsigned char* buff, unsigned char periodCode, unsigned char triggerMode, unsigned char triggerLevel, unsigned char delayUnit, int delay, long numData)
		{
			SET_CMND(buff,COMMAND_LEN_START_SAMPLING);
			SET_CMND(buff,COMMAND_CODE_START_SAMPLING);
			SET_CMND(buff,periodCode);		// bit7-4:1,2,5  bit3-0:unit 6:uS,7:10uS,8:100uS,9:mS,10:10mS
			SET_CMND(buff,triggerMode);
			SET_CMND(buff,triggerLevel);								// 8bit -128 to 127
			SET_CMND(buff,delayUnit);									// 2:uS, 3:mS, 4:S
			SET_CMND(buff,(unsigned char)((delay >> 8) & 0xFF));		//
			SET_CMND(buff,(unsigned char)(delay & 0xFF));
			SET_CMND(buff,(unsigned char)((numData >> 16) & 0xFF));		//
			SET_CMND(buff,(unsigned char)((numData >> 8) & 0xFF));		//
			SET_CMND(buff,(unsigned char)(numData & 0xFF));
		};
	static void MakeStopSampling(unsigned char* buff)
		{
			SET_CMND(buff,COMMAND_LEN_STOP_SAMPLING);
			SET_CMND(buff,COMMAND_CODE_STOP_SAMPLING);
		};

	static void MakeGetData(unsigned char* buff, unsigned char channel, unsigned char unit)
		{
			SET_CMND(buff,COMMAND_LEN_GET_DATA);
			SET_CMND(buff,COMMAND_CODE_GET_DATA);
			SET_CMND(buff,channel);
			SET_CMND(buff,unit);
		};
	static void MakeGetRange(unsigned char* buff, unsigned char channel, unsigned char unit)
		{
			SET_CMND(buff,COMMAND_LEN_GET_RANGE);
			SET_CMND(buff,COMMAND_CODE_GET_RANGE);
			SET_CMND(buff,channel);
			SET_CMND(buff,unit);
		};
	static void MakeGetPosition(unsigned char* buff, unsigned char channel, unsigned char unit)
		{
			SET_CMND(buff,COMMAND_LEN_GET_POSITION);
			SET_CMND(buff,COMMAND_CODE_GET_POSITION);
			SET_CMND(buff,channel);
			SET_CMND(buff,unit);
		};
	static void MakeGetTriggerLevel(unsigned char* buff, unsigned char channel, unsigned char unit)
		{
			SET_CMND(buff,COMMAND_LEN_GET_TRIGGER_LEVEL);
			SET_CMND(buff,COMMAND_CODE_GET_TRIGGER_LEVEL);
			SET_CMND(buff,channel);
			SET_CMND(buff,unit);
		};
	static void MakeGetSwitch(unsigned char* buff, unsigned char channel, unsigned char unit)
		{
			SET_CMND(buff,COMMAND_LEN_GET_SWITCH);
			SET_CMND(buff,COMMAND_CODE_GET_SWITCH);
			SET_CMND(buff,channel);
			SET_CMND(buff,unit);
		};
	static void MakeGetDCAC(unsigned char* buff, unsigned char channel, unsigned char unit)
		{
			SET_CMND(buff,COMMAND_LEN_GET_DCAC);
			SET_CMND(buff,COMMAND_CODE_GET_DCAC);
			SET_CMND(buff,channel);
			SET_CMND(buff,unit);
		};
	static void MakeGetSIGGND(unsigned char* buff, unsigned char channel, unsigned char unit)
		{
			SET_CMND(buff,COMMAND_LEN_GET_SIGGND);
			SET_CMND(buff,COMMAND_CODE_GET_SIGGND);
			SET_CMND(buff,channel);
			SET_CMND(buff,unit);
		};

//	--------------------------------------------------------------------------------------- ADD START --- V2.0
	static void MakeSetRange(unsigned char* buff, unsigned char channel, unsigned char val)
		{
			SET_CMND(buff,COMMAND_LEN_SET_RANGE);
			SET_CMND(buff,COMMAND_CODE_SET_RANGE);
			SET_CMND(buff,channel);
			SET_CMND(buff,val);
		};
	static void MakeSetPosition(unsigned char* buff, unsigned char channel, unsigned char val)
		{
			SET_CMND(buff,COMMAND_LEN_SET_POSITION);
			SET_CMND(buff,COMMAND_CODE_SET_POSITION);
			SET_CMND(buff,channel);
			SET_CMND(buff,val);
		};
	static void MakeSetTriggerMode(unsigned char* buff, unsigned char channel, unsigned char val)
		{
			SET_CMND(buff,COMMAND_LEN_SET_TRIGGER_MODE);
			SET_CMND(buff,COMMAND_CODE_SET_TRIGGER_MODE);
			SET_CMND(buff,channel);
			SET_CMND(buff,val);
		};
	static void MakeSetTriggerLevel(unsigned char* buff, unsigned char channel, unsigned char val)
		{
			SET_CMND(buff,COMMAND_LEN_SET_TRIGGER_LEVEL);
			SET_CMND(buff,COMMAND_CODE_SET_TRIGGER_LEVEL);
			SET_CMND(buff,channel);
			SET_CMND(buff,val);
		};
	static void MakeSetSwitch(unsigned char* buff, unsigned char channel, unsigned char val)
		{
			SET_CMND(buff,COMMAND_LEN_SET_SWITCH);
			SET_CMND(buff,COMMAND_CODE_SET_SWITCH);
			SET_CMND(buff,channel);
			SET_CMND(buff,val);
		};
//	--------------------------------------------------------------------------------------- ADD END ----- V2.0

	static void MakeClearAll(unsigned char* buff, unsigned char channel, unsigned char unit)
		{
			SET_CMND(buff,COMMAND_LEN_CLEAR_ALL);
			SET_CMND(buff,COMMAND_CODE_CLEAR_ALL);
			SET_CMND(buff,channel);
			SET_CMND(buff,unit);
		};
	static void MakeSetFastMode(unsigned char* buff, unsigned char channel, unsigned char unit)
		{
			SET_CMND(buff,COMMAND_LEN_SET_FAST_MODE);
			SET_CMND(buff,COMMAND_CODE_SET_FAST_MODE);
			SET_CMND(buff,channel);
			SET_CMND(buff,unit);
		};
	static void MakeSetSlowMode(unsigned char* buff, unsigned char channel, unsigned char unit)
		{
			SET_CMND(buff,COMMAND_LEN_SET_SLOW_MODE);
			SET_CMND(buff,COMMAND_CODE_SET_SLOW_MODE);
			SET_CMND(buff,channel);
			SET_CMND(buff,unit);
		};
	static void MakeGetNumData(unsigned char* buff, unsigned char channel, unsigned char unit)
		{
			SET_CMND(buff,COMMAND_LEN_GET_NUM_DATA);
			SET_CMND(buff,COMMAND_CODE_GET_NUM_DATA);
			SET_CMND(buff,channel);
			SET_CMND(buff,unit);
		};
	static void MakeGetFirstData(unsigned char* buff, unsigned char channel, unsigned char unit)
		{
			SET_CMND(buff,COMMAND_LEN_GET_FIRST_DATA);
			SET_CMND(buff,COMMAND_CODE_GET_FIRST_DATA);
			SET_CMND(buff,channel);
			SET_CMND(buff,unit);
		};
	static void MakeGetNextData(unsigned char* buff, unsigned char channel, unsigned char unit)
		{
			SET_CMND(buff,COMMAND_LEN_GET_NEXT_DATA);
			SET_CMND(buff,COMMAND_CODE_GET_NEXT_DATA);
			SET_CMND(buff,channel);
			SET_CMND(buff,unit);
		};

};
#endif

#if defined(__AVR_CTRL_INCLUDED__)
	void MakeReplyBadCommand(unsigned char* buff, unsigned char cmnd);
	void MakeReplyConfiguration(unsigned char* buff, unsigned char numChannels, unsigned char numADCs, unsigned int Vref, unsigned char slowestPeriodCode, unsigned char fastestPeriodCode, unsigned int buffSize);
	void MakeReplySettings(unsigned char* buff, unsigned char range1, unsigned char pos1, unsigned char sw1, unsigned char range2, unsigned char pos2, unsigned char sw2, unsigned char level, unsigned char val1, unsigned char val2);
	void MakeReplySetSettings(unsigned char* buff, unsigned char result);
	void MakeReplySampling(unsigned char* buff, unsigned char channel, long seq, unsigned char numData, unsigned char* dataBuff);
	void MakeReplyStopSampling(unsigned char* buff, unsigned int numData);

	void MakeReplyData(unsigned char* buff, unsigned char channel, unsigned char unit, unsigned char data);
	void MakeReplyRange(unsigned char* buff, unsigned char channel, unsigned char unit, unsigned char range, unsigned char val);
	void MakeReplyPosition(unsigned char* buff, unsigned char channel, unsigned char unit, unsigned char pos);
	void MakeReplyTriggerLevel(unsigned char* buff, unsigned char channel, unsigned char unit, unsigned char level);
	void MakeReplySwitch(unsigned char* buff, unsigned char channel, unsigned char unit, unsigned char s);
	void MakeReplyDCAC(unsigned char* buff, unsigned char channel, unsigned char unit, unsigned char flag);
	void MakeReplySIGGND(unsigned char* buff, unsigned char channel, unsigned char unit, unsigned char flag);

//	--------------------------------------------------------------------------------------- ADD START --- V2.0
	void MakeReplySetRange(unsigned char* buff, unsigned char channel, unsigned char val);
	void MakeReplySetPosition(unsigned char* buff, unsigned char channel, unsigned char val);
	void MakeReplySetTriggerMode(unsigned char* buff, unsigned char channel, unsigned char val);
	void MakeReplySetTriggerLevel(unsigned char* buff, unsigned char channel, unsigned char val);
	void MakeReplySetSwitch(unsigned char* buff, unsigned char channel, unsigned char val);
//	--------------------------------------------------------------------------------------- ADD END ----- V2.0

	void MakeReplyClearAll(unsigned char* buff, unsigned char channel, unsigned char unit);
	void MakeReplyFastMode(unsigned char* buff, unsigned char channel, unsigned char unit);
	void MakeReplySlowMode(unsigned char* buff, unsigned char channel, unsigned char unit);
	void MakeReplyNumData(unsigned char* buff, unsigned char channel, unsigned char unit, unsigned int num);
	void MakeReplyFirstData(unsigned char* buff, unsigned char channel, unsigned char unit, unsigned char data);
	void MakeReplyNextData(unsigned char* buff, unsigned char channel, unsigned char unit, unsigned char data);
#endif

#endif // CMNDMSG_H_INCLUDED
