/*

					Oscilloscope AVR

			Copyrights 2008,  Yoshi Hiroshi

		Control Block - ATmega644P/ATmega324P

*/

#include <avr/io.h>
#include <avr/interrupt.h>

#include "OscilloCtrl.h"
#include "Spi.h"
#include "Buffer.h"
#include "Uart.h"
#include "Sampling.h"
#include "AdcManager.h"
#include "Command.h"

volatile long test;
volatile unsigned char usartRxCompleteFlag;
volatile unsigned char usartUDREmptyFlag;

volatile unsigned char stopRequestFlag;
volatile unsigned char slowSamplingIntFlag;
volatile unsigned char fastSamplingCompleteIntFlag;

//**********************************************************
//		USART0_TX_vect
//**********************************************************
ISR(USART0_RX_vect)
{
	usartRxData = UART_DATA_REGISTER;
	usartRxCompleteFlag = INTERRUPT_OCCURED;
}

//**********************************************************
//		USART0_UDRE_vect
//**********************************************************
ISR(USART0_UDRE_vect)
{
	UART_TRL_STATUS_B &= ~_BV(UART_TXBUFEMPTY_INT_ENABLE_BIT);
	usartUDREmptyFlag = INTERRUPT_OCCURED;
}


//**********************************************************
//		main
//**********************************************************
int main()
{
	usartRxCompleteFlag = INTERRUPT_NONE;
	usartUDREmptyFlag = INTERRUPT_NONE;
	slowSamplingIntFlag = INTERRUPT_NONE;
	fastSamplingCompleteIntFlag = INTERRUPT_NONE;

	for(volatile long waitForStartup = 0; waitForStartup < 120000; waitForStartup++);
	Sampling_Init();
	SPI_Init();
	InitBuff();
	InitUART();
	DEBUG_PORT |= _BV(DEBUG_BIT);
	CheckConfiguration();
	sei();

	while(1)
	{
		if(usartRxCompleteFlag == INTERRUPT_OCCURED)
		{
			usartRxCompleteFlag = INTERRUPT_NONE;
			RecieveDataReady();
		}
		if(usartUDREmptyFlag == INTERRUPT_OCCURED)
		{
			usartUDREmptyFlag = INTERRUPT_NONE;
			TransmitDataEmpty();
		}

		if(!IsUsedRxBuffEmpty())
		{
			DoCommand();
		}

		if(slowSamplingIntFlag == INTERRUPT_OCCURED)	// Slow Sampling
		{
			slowSamplingIntFlag = INTERRUPT_NONE;
			SlowSamplingDataTransfer();
		}

		if(fastSamplingCompleteIntFlag == INTERRUPT_OCCURED)		// Fast Sampling
		{
			fastSamplingCompleteIntFlag = INTERRUPT_NONE;
			FastSamplingDataTransfer();
		}
	}
}


