/*

					Oscilloscope AVR

			Copyrights 2008,  Yoshi Hiroshi

		Control Block - ATmega644P/ATmega324P

*/

#if !defined(__SAMPLING_MANAGER_HEADER_INCLUDED__)
#define __SAMPLING_MANAGER_HEADER_INCLUDED__

/*
		DEBUG PORT & PIN
*/
#define DEBUG_PORT					PORTB
#define DEBUG_PIN					PINB
#define DEBUG_BIT						PB3		// ADC8Œ

/*
------------------------------------------------------------
		Trigger and Sampling Control
------------------------------------------------------------
*/
#define TRIGGER_INT_PORT			PORTD
#define TRIGGER_INT_MASK			0x0C
#define TRIGGER_INT_CH1					PD2		// INT0
#define TRIGGER_INT_CH2					PD3		// INT1

/*
------------------------------------------------------------
		Timing Generator Control-Port
------------------------------------------------------------
*/
#define TIMING_GEN_CMND_PORT		PORTD
#define TIMING_GEN_CMND_DDR			DDRD
#define TIMING_GEN_CMND_MASK		0xF0
#define TIMING_GEN_COMMAND_POS			4	

#define TIMING_GEN_PARAM_PORT		PORTB
#define TIMING_GEN_PARAM_DDR		DDRB
#define TIMING_GEN_PARAM_MASK		0x03
#define TIMING_GEN_PARAM_POS			0

#define TIMING_GEN_INT_PORT			PORTB
#define TIMING_GEN_INT_MASK			0x04
#define TIMING_GEN_INT_ADCSTART			PB2		// INT2
/*
------------------------------------------------------------
		Timing Generator Controls
------------------------------------------------------------
*/
#define TIMING_GEN_DO_COMMAND		7

#define TIMING_GEN_CMND_SET_ADCS		4
#define TIMING_GEN_CMND_START_ALL		5
#define TIMING_GEN_CMND_START_x1u		6		// 0x06
#define TIMING_GEN_CMND_START_x10u		7		// 0x07
#define TIMING_GEN_CMND_START_x100u		0		// 0x08
#define TIMING_GEN_CMND_START_x1m		1		// 0x09
#define TIMING_GEN_CMND_START_x10m		2		// 0x0A
#define TIMING_GEN_CMND_START_x100m		3		// 0x0B

#define TIMING_GEN_PARAM_ADC_1			0
#define TIMING_GEN_PARAM_ADC_2			1
#define TIMING_GEN_PARAM_ADC_4			2
#define TIMING_GEN_PARAM_ADC_8			3

#define TIMING_GEN_PARAM_PERIOD_1		1
#define TIMING_GEN_PARAM_PERIOD_2		2
#define TIMING_GEN_PARAM_PERIOD_5		3

/*
------------------------------------------------------------
					Local Working Strage
------------------------------------------------------------
*/
extern volatile unsigned char triggerMode;
extern volatile char triggerLevel;
extern volatile unsigned char triggerEdge;
extern volatile unsigned char triggerChannel;
extern volatile long numSampleData;
extern volatile unsigned char samplePeriodCode;
extern volatile unsigned char delayUnit;
extern volatile unsigned int delayTime;


/*
------------------------------------------------------------
					Local Functions
------------------------------------------------------------
*/
void Sampling_Init();

void SamplingStart();
void SamplingStop(unsigned char forced);

void SlowSamplingDataTransfer();
void FastSamplingDataTransfer();

void SetTimingGeneratorCommand(unsigned char cmnd, unsigned char param, unsigned char cont);
unsigned char GetNumADCsCode(unsigned char num);
int GetPeriodCodeIndex(unsigned char periodCode);
void SetTimingGenerator(unsigned char period, unsigned char numAdc);

#endif

