/*

					Oscilloscope AVR

			Copyrights 2008,  Yoshi Hiroshi

		Control Block - ATmega644P/ATmega324P

*/

#include "OscilloCtrl.h"


unsigned char periodCode[] =
{
	TIMEBASE_CODE_2uS,			// 0
	TIMEBASE_CODE_5uS,			// 1
	TIMEBASE_CODE_10uS,			// 2
	TIMEBASE_CODE_20uS,			// 3
	TIMEBASE_CODE_50uS,			// 4
	TIMEBASE_CODE_100uS,		// 5
	TIMEBASE_CODE_200uS,		// 6
	TIMEBASE_CODE_500uS,		// 7
	TIMEBASE_CODE_1mS,			// 8
	TIMEBASE_CODE_2mS,			// 9
	TIMEBASE_CODE_5mS,			// 10
	TIMEBASE_CODE_10mS,			// 11
	TIMEBASE_CODE_20mS,			// 12
	TIMEBASE_CODE_50mS,			// 13
	TIMEBASE_CODE_100mS,		// 14
	TIMEBASE_CODE_200mS,		// 15
	TIMEBASE_CODE_500mS			// 16
};

//	Autõ^CAEgl
long autoTriggerRemain[] =			// 1 sec
{
	500000,		// TIMEBASE_CODE_2uS,
	200000,		// TIMEBASE_CODE_5uS,
	100000,		// TIMEBASE_CODE_10uS,
	50000,		// TIMEBASE_CODE_20uS,
	20000,		// TIMEBASE_CODE_50uS,
	10000,		// TIMEBASE_CODE_100uS,
	5000,		// TIMEBASE_CODE_200uS,
	2000,		// TIMEBASE_CODE_500uS,
	1000,		// TIMEBASE_CODE_1mS,
	500,		// TIMEBASE_CODE_2mS,
	200,		// TIMEBASE_CODE_5mS,
	100,		// TIMEBASE_CODE_10mS,
	50,			// TIMEBASE_CODE_20mS,
	20,			// TIMEBASE_CODE_50mS,
	10,			// TIMEBASE_CODE_100mS,
	5,			// TIMEBASE_CODE_200mS,
	2			// TIMEBASE_CODE_500mS
};

//	Normal̃^CAEgl
long autoTriggerInfinite[] =		// 10 or 30 sec
{
	5000000,	// TIMEBASE_CODE_2uS,		// 10 sec
	2000000,	// TIMEBASE_CODE_5uS,		// 10 sec
	1000000,	// TIMEBASE_CODE_10uS,		// 10 sec
	500000,		// TIMEBASE_CODE_20uS,		// 10 sec
	200000,		// TIMEBASE_CODE_50uS,		// 10 sec
	100000,		// TIMEBASE_CODE_100uS,		// 10 sec
	150000,		// TIMEBASE_CODE_200uS,		// 30 sec
	60000,		// TIMEBASE_CODE_500uS,		// 30 sec
	30000,		// TIMEBASE_CODE_1mS,		// 30 sec
	15000,		// TIMEBASE_CODE_2mS,		// 30 sec
	6000,		// TIMEBASE_CODE_5mS,		// 30 sec
	3000,		// TIMEBASE_CODE_10mS,		// 30 sec
	1500,		// TIMEBASE_CODE_20mS,		// 30 sec
	600,		// TIMEBASE_CODE_50mS,		// 30 sec
	300,		// TIMEBASE_CODE_100mS,		// 30 sec
	150,		// TIMEBASE_CODE_200mS,		// 30sec
	60			// TIMEBASE_CODE_500mS		// 30 sec
};



//**********************************************************
//					GetPeriodCodeIndex
//**********************************************************
int GetPeriodCodeIndex(unsigned char pc)
{
	int result = -1;
	for(int index = 0; index < 18; index++)
	{
		if(periodCode[index] == pc)
		{
			result = index;
			break;
		}
	}
	return result;
}

//**********************************************************
//					GetAutoTriggerRemain
//**********************************************************
long GetAutoTriggerRemain(int pIndex)
{
	return autoTriggerRemain[pIndex];
}

//**********************************************************
//					GetAutoTriggerInfinite
//**********************************************************
long GetAutoTriggerInfinite(int pIndex)
{
	return autoTriggerInfinite[pIndex];
}

//**********************************************************
//					TimebaseCodeDivision
//**********************************************************
long TimebaseCodeDivision(unsigned char dividendCode, unsigned char divisorCode)
{
	long result = 0;
	int exp = (dividendCode & 0x0F) - (divisorCode & 0x0F);
	long fraction = (dividendCode >> 4) & 0x0F;
	if(exp >= 0)
	{
		while(exp > 0)
		{
			fraction *= 10;
			exp--;
		}
		result = fraction / ((divisorCode >> 4) & 0x0F);
	}
	return result;
}

//**********************************************************
//					TimebaseDivision
//**********************************************************
long TimebaseDivision(long dividend, unsigned char divisorCode)
{
	long result = dividend;
	int exp = (divisorCode & 0x0F) - 6;
	while(exp > 0)
	{
		result /= 10;
		exp--;
	}
	result = result / ((divisorCode >> 4) & 0x0F);
	return result;
}
