/*

					Oscilloscope AVR

			Copyrights 2008,  Yoshi Hiroshi

		Control Block - ATmega644P/ATmega324P
*/

#include <avr/io.h>
#include <util/atomic.h>
#include "OscilloCtrl.h"
#include "Buffer.h"
#include "Uart.h"


char usartRxData;
rxBuffType* currRxBuff;
volatile unsigned int rxCount;
volatile unsigned char zeroCount;
txBuffType* currTxBuff;
volatile unsigned int txCount;


//**********************************************************
//		Init UART
//**********************************************************
//
//----------------------------------------------------------
void InitUART()
{
	UART_BAUD_RATE_REGISTER = USART_BAUDRATE;
	UART_TRL_STATUS_A = _BV(UART_2X_SPEED_BIT);
	UART_TRL_STATUS_C = UART_CHARACTER_SIZE;
	UART_TRL_STATUS_B = _BV(UART_RXCMP_INT_ENABLE_BIT) | 
//						_BV(UART_TXBUFEMPTY_INT_ENABLE_BIT) | 
						_BV(UART_TRANSMITTER_ENABLE_BIT) | 
						_BV(UART_RECIEVER_ENABLE_BIT);
	UART_TRANSMITTER_PORT_DIRECTION;

	currRxBuff = 0;
	zeroCount = 0;
	currTxBuff = 0;
}

//**********************************************************
//		Send Data in Transmit Buffer
//**********************************************************
//
//----------------------------------------------------------
void SendTxBuff(txBuffType* ptr)
{
	PutUsedTxBuff(ptr);
	if(currTxBuff == 0)
	{
		while((UART_TRL_STATUS_A & _BV(UART_DATA_EMPTY_BIT)) == 0);
		currTxBuff = GetUsedTxBuff(GET_BUFFER_NOWAIT);
		txCount = 0;
		UART_DATA_REGISTER = currTxBuff->buff[txCount++];
		UART_TRL_STATUS_B |= _BV(UART_TXBUFEMPTY_INT_ENABLE_BIT);
	}
}

//**********************************************************
//		Transmit Data Buffer Empty Interrupt Support
//**********************************************************
//
//----------------------------------------------------------
void TransmitDataEmpty()
{
	if(currTxBuff != 0)
	{
		UART_DATA_REGISTER = currTxBuff->buff[txCount++];
		UART_TRL_STATUS_B |= _BV(UART_TXBUFEMPTY_INT_ENABLE_BIT);
		if(txCount > currTxBuff->buff[0])
		{
			PutFreeTxBuff(currTxBuff);
			currTxBuff = GetUsedTxBuff(GET_BUFFER_NOWAIT);
			txCount = 0;
		}
	}
}

//**********************************************************
//		Reciieve Data Ready Interrupt Support Routine
//**********************************************************
//
//----------------------------------------------------------
void RecieveDataReady()
{
	txBuffType* sPtr;
	unsigned char cmnd;
	if(currRxBuff == 0)
	{
		currRxBuff = GetFreeRxBuff(GET_BUFFER_WAIT);
		rxCount = 0;
	}
	if(rxCount == 0)
	{
		if((usartRxData < RECIEVE_BUFFER_SIZE) && (usartRxData > 0))
		{
			if(usartRxData != 0)
			{
				currRxBuff->buff[0] = usartRxData;
				rxCount = 1;
			} else {	// Ignore Null Length Command
//				rxCount = 0;
			}
		} else {	// Ignore Size Error
//			rxCount = 0;
		}
	} else {
		currRxBuff->buff[rxCount++] = usartRxData;
		if(rxCount == 2)
		{
			cmnd = currRxBuff->buff[1];
			switch(cmnd)
			{
			case COMMAND_CODE_RESET_ALL:	// GetConfiguration
				if((currRxBuff->buff[0] != COMMAND_LEN_RESET_ALL) &&
					(currRxBuff->buff[0] != 1)) rxCount = 0;
				break;
			case COMMAND_CODE_GET_CONFIGURATION:	// GetConfiguration
				if(currRxBuff->buff[0] != COMMAND_LEN_GET_CONFIGURATION) rxCount = 0;
				break;
//			case COMMAND_CODE_GET_SETTINGS:			// GetSettings
//				if(currRxBuff->buff[0] != COMMAND_LEN_GET_SETTINGS) rxCount = 0;
//				break;
			case COMMAND_CODE_SET_SETTINGS:			// GetSettings
				if(currRxBuff->buff[0] != COMMAND_LEN_SET_SETTINGS) rxCount = 0;
				break;
			case COMMAND_CODE_START_SAMPLING:		// StartSampling
				if(currRxBuff->buff[0] != COMMAND_LEN_START_SAMPLING) rxCount = 0;
				break;
			case COMMAND_CODE_STOP_SAMPLING:		// StopSampling
				if(currRxBuff->buff[0] != COMMAND_LEN_STOP_SAMPLING) rxCount = 0;
				break;

			case COMMAND_CODE_GET_DATA:				// GetData
				if(currRxBuff->buff[0] != COMMAND_LEN_GET_DATA) rxCount = 0;
				break;
//			case COMMAND_CODE_GET_RANGE:			// GetRange
//				if(currRxBuff->buff[0] != COMMAND_LEN_GET_RANGE) rxCount = 0;
//				break;
//			case COMMAND_CODE_GET_POSITION:			// GetPosition
//				if(currRxBuff->buff[0] != COMMAND_LEN_GET_POSITION) rxCount = 0;
//				break;
//			case COMMAND_CODE_GET_TRIGGER_LEVEL:	// GetTriggerLevel
//				if(currRxBuff->buff[0] != COMMAND_LEN_GET_TRIGGER_LEVEL) rxCount = 0;
//				break;
//			case COMMAND_CODE_GET_SWITCH:			// GetSwitch
//				if(currRxBuff->buff[0] != COMMAND_LEN_GET_SWITCH) rxCount = 0;
//				break;
//			case COMMAND_CODE_GET_DCAC:				// GetACDC
//				if(currRxBuff->buff[0] != COMMAND_LEN_GET_DCAC) rxCount = 0;
//				break;
//			case COMMAND_CODE_GET_SIGGND:			// GetSIGGND
//				if(currRxBuff->buff[0] != COMMAND_LEN_GET_SIGGND) rxCount = 0;
//				break;

			case COMMAND_CODE_SET_RANGE:			// SetRange
				if(currRxBuff->buff[0] != COMMAND_LEN_SET_RANGE) rxCount = 0;
				break;
			case COMMAND_CODE_SET_POSITION:			// SetPosition
				if(currRxBuff->buff[0] != COMMAND_LEN_SET_POSITION) rxCount = 0;
				break;
			case COMMAND_CODE_SET_TRIGGER_LEVEL:	// SetTriggerLevel
				if(currRxBuff->buff[0] != COMMAND_LEN_SET_TRIGGER_LEVEL) rxCount = 0;
				break;
			case COMMAND_CODE_SET_SWITCH:			// SetSwitch
				if(currRxBuff->buff[0] != COMMAND_LEN_SET_SWITCH) rxCount = 0;
				break;

			case COMMAND_CODE_CLEAR_ALL:			// ClearAll
				if(currRxBuff->buff[0] != COMMAND_LEN_CLEAR_ALL) rxCount = 0;
				break;
//			case COMMAND_CODE_SET_FAST_MODE:		// SetFastMode
//				if(currRxBuff->buff[0] != COMMAND_LEN_SET_FAST_MODE) rxCount = 0;
//				break;
//			case COMMAND_CODE_SET_SLOW_MODE:		// SetSlowMode
//				if(currRxBuff->buff[0] != COMMAND_LEN_SET_SLOW_MODE) rxCount = 0;
//				break;
			case COMMAND_CODE_GET_NUM_DATA:			// GetNumData
				if(currRxBuff->buff[0] != COMMAND_LEN_GET_NUM_DATA) rxCount = 0;
				break;
			case COMMAND_CODE_GET_FIRST_DATA:		// GetFirstData
				if(currRxBuff->buff[0] != COMMAND_LEN_GET_FIRST_DATA) rxCount = 0;
				break;
			case COMMAND_CODE_GET_NEXT_DATA:		// GetNextData
				if(currRxBuff->buff[0] != COMMAND_LEN_GET_NEXT_DATA) rxCount = 0;
				break;
			default:
				rxCount = 0;
				break;
			}
		}
		if(rxCount == 0)
		{
			sPtr = GetFreeTxBuff(GET_BUFFER_WAIT);
			MakeReplyBadCommand(sPtr->buff, currRxBuff->buff[1]);
			SendTxBuff(sPtr);
		}
		if(rxCount > currRxBuff->buff[0])
		{
			PutUsedRxBuff(currRxBuff);
			currRxBuff = 0;
			rxCount = 0;
		}
	}

	if(usartRxData == 0)
	{
		zeroCount++;
		if(zeroCount >= 9)
		{
			rxCount = 0;
			stopRequestFlag = STOP_REQUEST_ON;
		}
	}
	else
	{
		zeroCount = 0;
	}
}


